//
//  NSColor+Rainbow.swift
//  Rainbow NSColor Extension
//
//  Created by Reid Gravelle on 2015-04-02.
//  Copyright (c) 2015 Northern Realities Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

import AppKit

extension NSColor {
    /**
    Returns a NSColor object representing the color Acid Green, whose RBG values are (176, 191, 26), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func acidGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 191.0/255.0, blue: 26.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Aero, whose RBG values are (124, 185, 232), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aeroColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 185.0/255.0, blue: 232.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Aero Blue, whose RBG values are (201, 255, 229), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aeroBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 201.0/255.0, green: 255.0/255.0, blue: 229.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color African Violet, whose RBG values are (178, 132, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func africanVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 132.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Air Force Blue (RAF), whose RBG values are (93, 138, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func airForceBlueRAFColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 93.0/255.0, green: 138.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Air Force Blue (USAF), whose RBG values are (0, 48, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func airForceBlueUSAFColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 48.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Air Superiority Blue, whose RBG values are (114, 160, 193), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func airSuperiorityBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 114.0/255.0, green: 160.0/255.0, blue: 193.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Alabama Crimson, whose RBG values are (175, 0, 42), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func alabamaCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 175.0/255.0, green: 0.0/255.0, blue: 42.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Alice Blue, whose RBG values are (240, 248, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aliceBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 248.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Alizarin Crimson, whose RBG values are (227, 38, 54), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func alizarinCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 38.0/255.0, blue: 54.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Alloy Orange, whose RBG values are (196, 98, 16), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func alloyOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 196.0/255.0, green: 98.0/255.0, blue: 16.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Almond (Crayola), whose RBG values are (239, 222, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func almondCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 222.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amaranth, whose RBG values are (229, 43, 80), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amaranthColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 43.0/255.0, blue: 80.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amaranth Deep Purple, whose RBG values are (171, 39, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amaranthDeepPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 171.0/255.0, green: 39.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amaranth Pink, whose RBG values are (241, 156, 187), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amaranthPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 241.0/255.0, green: 156.0/255.0, blue: 187.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amaranth Purple, whose RBG values are (171, 39, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amaranthPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 171.0/255.0, green: 39.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amaranth Red, whose RBG values are (211, 33, 45), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amaranthRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 211.0/255.0, green: 33.0/255.0, blue: 45.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amazon, whose RBG values are (59, 122, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amazonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 59.0/255.0, green: 122.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber, whose RBG values are (255, 191, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amberColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 191.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber (SAE/ECE), whose RBG values are (255, 126, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amberSAEECEColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 126.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 100, whose RBG values are (255, 236, 179), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 236.0/255.0, blue: 179.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 200, whose RBG values are (255, 224, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 224.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 300, whose RBG values are (255, 213, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 213.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 400, whose RBG values are (255, 202, 40), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 202.0/255.0, blue: 40.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 50, whose RBG values are (255, 248, 225), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 248.0/255.0, blue: 225.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 500, whose RBG values are (255, 193, 7), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 193.0/255.0, blue: 7.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 600, whose RBG values are (255, 179, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 179.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 700, whose RBG values are (255, 160, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 160.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 800, whose RBG values are (255, 143, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 143.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber 900, whose RBG values are (255, 111, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amber900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 111.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber A100, whose RBG values are (255, 229, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amberA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 229.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber A200, whose RBG values are (255, 215, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amberA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 215.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber A400, whose RBG values are (255, 196, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amberA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 196.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amber A700, whose RBG values are (255, 171, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amberA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 171.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color American Rose, whose RBG values are (255, 3, 62), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func americanRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 3.0/255.0, blue: 62.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Amethyst, whose RBG values are (153, 102, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func amethystColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 102.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Android Green, whose RBG values are (164, 198, 57), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func androidGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 164.0/255.0, green: 198.0/255.0, blue: 57.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Anti-Flash White, whose RBG values are (242, 243, 244), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func antiFlashWhiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 242.0/255.0, green: 243.0/255.0, blue: 244.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Antique Brass (Crayola), whose RBG values are (205, 149, 117), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func antiqueBrassCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 149.0/255.0, blue: 117.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Antique Bronze, whose RBG values are (102, 93, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func antiqueBronzeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 93.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Antique Fuchsia, whose RBG values are (145, 92, 131), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func antiqueFuchsiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 145.0/255.0, green: 92.0/255.0, blue: 131.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Antique Ruby, whose RBG values are (132, 27, 45), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func antiqueRubyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 132.0/255.0, green: 27.0/255.0, blue: 45.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Antique White, whose RBG values are (250, 235, 215), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func antiqueWhiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 235.0/255.0, blue: 215.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ao (English), whose RBG values are (0, 128, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aoEnglishColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 128.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Apple Green, whose RBG values are (141, 182, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func appleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 141.0/255.0, green: 182.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Apricot, whose RBG values are (251, 206, 177), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func apricotColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 206.0/255.0, blue: 177.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Apricot (Crayola), whose RBG values are (253, 217, 181), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func apricotCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 217.0/255.0, blue: 181.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Aqua, whose RBG values are (0, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aquaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Aquamarine, whose RBG values are (127, 255, 212), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aquamarineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 127.0/255.0, green: 255.0/255.0, blue: 212.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Aquamarine (Crayola), whose RBG values are (120, 219, 226), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aquamarineCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 120.0/255.0, green: 219.0/255.0, blue: 226.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Arctic Lime, whose RBG values are (208, 255, 20), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func arcticLimeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 208.0/255.0, green: 255.0/255.0, blue: 20.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Army Green, whose RBG values are (75, 83, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func armyGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 75.0/255.0, green: 83.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Arsenic, whose RBG values are (59, 68, 75), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func arsenicColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 59.0/255.0, green: 68.0/255.0, blue: 75.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Artichoke, whose RBG values are (143, 151, 121), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func artichokeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 143.0/255.0, green: 151.0/255.0, blue: 121.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Arylide Yellow, whose RBG values are (233, 214, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func arylideYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 214.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ash Grey, whose RBG values are (178, 190, 181), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ashGreyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 190.0/255.0, blue: 181.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Asparagus (Crayola), whose RBG values are (135, 169, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func asparagusCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 135.0/255.0, green: 169.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Atomic Tangerine (Crayola), whose RBG values are (255, 164, 116), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func atomicTangerineCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 164.0/255.0, blue: 116.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Atomic Tangerine, whose RBG values are (255, 153, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func atomicTangerineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 153.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Auburn, whose RBG values are (165, 42, 42), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func auburnColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 165.0/255.0, green: 42.0/255.0, blue: 42.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Aureolin, whose RBG values are (253, 238, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func aureolinColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 238.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color AuroMetalSaurus, whose RBG values are (110, 127, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func auroMetalSaurusColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 110.0/255.0, green: 127.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Avocado, whose RBG values are (86, 130, 3), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func avocadoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 86.0/255.0, green: 130.0/255.0, blue: 3.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Azure, whose RBG values are (0, 127, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func azureColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 127.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Azure (Web Color), whose RBG values are (240, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func azureWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Azure Mist, whose RBG values are (240, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func azureMistColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Azureish White, whose RBG values are (219, 233, 244), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func azureishWhiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 219.0/255.0, green: 233.0/255.0, blue: 244.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color B'dazzled Blue, whose RBG values are (46, 88, 148), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bdazzledBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 46.0/255.0, green: 88.0/255.0, blue: 148.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Baby Blue, whose RBG values are (137, 207, 240), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func babyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 137.0/255.0, green: 207.0/255.0, blue: 240.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Baby Blue Eyes, whose RBG values are (161, 202, 241), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func babyBlueEyesColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 161.0/255.0, green: 202.0/255.0, blue: 241.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Baby Pink, whose RBG values are (244, 194, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func babyPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 194.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Baby Powder, whose RBG values are (254, 254, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func babyPowderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 254.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Baker-Miller Pink, whose RBG values are (255, 145, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bakerMillerPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 145.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ball Blue, whose RBG values are (33, 171, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ballBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 33.0/255.0, green: 171.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Banana Mania (Crayola), whose RBG values are (250, 231, 181), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bananaManiaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 231.0/255.0, blue: 181.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Banana Yellow, whose RBG values are (255, 225, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bananaYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 225.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bangladesh Green, whose RBG values are (0, 106, 78), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bangladeshGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 106.0/255.0, blue: 78.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Barbie Pink, whose RBG values are (224, 33, 138), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func barbiePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 33.0/255.0, blue: 138.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Barn Red, whose RBG values are (124, 10, 2), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func barnRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 10.0/255.0, blue: 2.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Battleship Grey, whose RBG values are (132, 132, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func battleshipGreyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 132.0/255.0, green: 132.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bazaar, whose RBG values are (152, 119, 123), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bazaarColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 119.0/255.0, blue: 123.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Beau Blue, whose RBG values are (188, 212, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func beauBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 188.0/255.0, green: 212.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Beaver (Crayola), whose RBG values are (159, 129, 112), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func beaverCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 159.0/255.0, green: 129.0/255.0, blue: 112.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Beige, whose RBG values are (245, 245, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func beigeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 245.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Big Dip O’ruby, whose RBG values are (156, 37, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bigDipOrubyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 156.0/255.0, green: 37.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bisque, whose RBG values are (255, 228, 196), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bisqueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 228.0/255.0, blue: 196.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bistre, whose RBG values are (61, 43, 31), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bistreColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 61.0/255.0, green: 43.0/255.0, blue: 31.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bistre Brown, whose RBG values are (150, 113, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bistreBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 113.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bitter Lemon, whose RBG values are (202, 224, 13), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bitterLemonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 202.0/255.0, green: 224.0/255.0, blue: 13.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bitter Lime, whose RBG values are (191, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bitterLimeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 191.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bittersweet, whose RBG values are (254, 111, 94), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bittersweetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 111.0/255.0, blue: 94.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bittersweet (Crayola), whose RBG values are (253, 124, 110), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bittersweetCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 124.0/255.0, blue: 110.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bittersweet Shimmer, whose RBG values are (191, 79, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bittersweetShimmerColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 191.0/255.0, green: 79.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Black Bean, whose RBG values are (61, 12, 2), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blackBeanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 61.0/255.0, green: 12.0/255.0, blue: 2.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Black Leather Jacket, whose RBG values are (37, 53, 41), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blackLeatherJacketColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 37.0/255.0, green: 53.0/255.0, blue: 41.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Black Olive, whose RBG values are (59, 60, 54), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blackOliveColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 59.0/255.0, green: 60.0/255.0, blue: 54.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blanched Almond, whose RBG values are (255, 235, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blanchedAlmondColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 235.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blast-Off Bronze, whose RBG values are (165, 113, 100), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blastOffBronzeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 165.0/255.0, green: 113.0/255.0, blue: 100.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bleu De France, whose RBG values are (49, 140, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bleuDeFranceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 49.0/255.0, green: 140.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blizzard Blue (Crayola), whose RBG values are (172, 229, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blizzardBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 172.0/255.0, green: 229.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blond, whose RBG values are (250, 240, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blondColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 240.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue (Crayola), whose RBG values are (31, 117, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 31.0/255.0, green: 117.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue (Munsell), whose RBG values are (0, 147, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueMunsellColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 147.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue (NCS), whose RBG values are (0, 135, 189), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueNCSColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 135.0/255.0, blue: 189.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue (Pantone), whose RBG values are (0, 24, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bluePantoneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 24.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue (Pigment), whose RBG values are (51, 51, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bluePigmentColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 51.0/255.0, green: 51.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue (RYB), whose RBG values are (2, 71, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueRYBColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 2.0/255.0, green: 71.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 100, whose RBG values are (187, 222, 251), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 187.0/255.0, green: 222.0/255.0, blue: 251.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 200, whose RBG values are (144, 202, 249), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 144.0/255.0, green: 202.0/255.0, blue: 249.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 300, whose RBG values are (100, 181, 246), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 100.0/255.0, green: 181.0/255.0, blue: 246.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 400, whose RBG values are (66, 165, 245), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 66.0/255.0, green: 165.0/255.0, blue: 245.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 50, whose RBG values are (227, 242, 253), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 242.0/255.0, blue: 253.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 500, whose RBG values are (33, 150, 243), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 33.0/255.0, green: 150.0/255.0, blue: 243.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 600, whose RBG values are (30, 136, 229), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 30.0/255.0, green: 136.0/255.0, blue: 229.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 700, whose RBG values are (25, 118, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 25.0/255.0, green: 118.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 800, whose RBG values are (21, 101, 192), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 21.0/255.0, green: 101.0/255.0, blue: 192.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue 900, whose RBG values are (13, 71, 161), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blue900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 13.0/255.0, green: 71.0/255.0, blue: 161.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue A100, whose RBG values are (130, 177, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 130.0/255.0, green: 177.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue A200, whose RBG values are (68, 138, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 68.0/255.0, green: 138.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue A400, whose RBG values are (41, 121, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 41.0/255.0, green: 121.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue A700, whose RBG values are (41, 98, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 41.0/255.0, green: 98.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Bell (Crayola), whose RBG values are (162, 162, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueBellCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 162.0/255.0, green: 162.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Green (Crayola), whose RBG values are (13, 152, 186), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 13.0/255.0, green: 152.0/255.0, blue: 186.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 100, whose RBG values are (207, 216, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 207.0/255.0, green: 216.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 200, whose RBG values are (176, 190, 197), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 190.0/255.0, blue: 197.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 300, whose RBG values are (144, 164, 174), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 144.0/255.0, green: 164.0/255.0, blue: 174.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 400, whose RBG values are (120, 144, 156), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 120.0/255.0, green: 144.0/255.0, blue: 156.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 50, whose RBG values are (236, 239, 241), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 236.0/255.0, green: 239.0/255.0, blue: 241.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 500, whose RBG values are (96, 125, 139), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 96.0/255.0, green: 125.0/255.0, blue: 139.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 600, whose RBG values are (84, 110, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 84.0/255.0, green: 110.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 700, whose RBG values are (69, 90, 100), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 69.0/255.0, green: 90.0/255.0, blue: 100.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 800, whose RBG values are (55, 71, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 55.0/255.0, green: 71.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Grey 900, whose RBG values are (38, 50, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrey900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 38.0/255.0, green: 50.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Lagoon, whose RBG values are (94, 147, 161), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueLagoonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 94.0/255.0, green: 147.0/255.0, blue: 161.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Violet (Crayola), whose RBG values are (115, 102, 189), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueVioletCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 115.0/255.0, green: 102.0/255.0, blue: 189.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Sapphire, whose RBG values are (18, 97, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueSapphireColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 18.0/255.0, green: 97.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue Yonder, whose RBG values are (80, 114, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueYonderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 80.0/255.0, green: 114.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue-Gray, whose RBG values are (102, 153, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 153.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue-Magenta Violet, whose RBG values are (85, 53, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueMagentaVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 85.0/255.0, green: 53.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blue-Violet, whose RBG values are (138, 43, 226), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 138.0/255.0, green: 43.0/255.0, blue: 226.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blueberry, whose RBG values are (79, 134, 247), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blueberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 79.0/255.0, green: 134.0/255.0, blue: 247.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bluebonnet, whose RBG values are (28, 28, 240), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bluebonnetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 28.0/255.0, green: 28.0/255.0, blue: 240.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Blush (Crayola), whose RBG values are (222, 93, 131), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func blushCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 93.0/255.0, blue: 131.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bole, whose RBG values are (121, 68, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func boleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 121.0/255.0, green: 68.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bondi Blue, whose RBG values are (0, 149, 182), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bondiBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 149.0/255.0, blue: 182.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bone, whose RBG values are (227, 218, 201), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func boneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 218.0/255.0, blue: 201.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Boston University Red, whose RBG values are (204, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bostonUniversityRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bottle Green, whose RBG values are (0, 106, 78), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bottleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 106.0/255.0, blue: 78.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Boysenberry, whose RBG values are (135, 50, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func boysenberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 135.0/255.0, green: 50.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brandeis Blue, whose RBG values are (0, 112, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brandeisBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 112.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brass, whose RBG values are (181, 166, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brassColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 181.0/255.0, green: 166.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brick Red (Crayola), whose RBG values are (203, 65, 84), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brickRedCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 203.0/255.0, green: 65.0/255.0, blue: 84.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Cerulean, whose RBG values are (29, 172, 214), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightCeruleanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 29.0/255.0, green: 172.0/255.0, blue: 214.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Green, whose RBG values are (102, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Lavender, whose RBG values are (191, 148, 228), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 191.0/255.0, green: 148.0/255.0, blue: 228.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Lilac, whose RBG values are (216, 145, 239), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightLilacColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 216.0/255.0, green: 145.0/255.0, blue: 239.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Maroon, whose RBG values are (195, 33, 72), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightMaroonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 195.0/255.0, green: 33.0/255.0, blue: 72.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Navy Blue, whose RBG values are (25, 116, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightNavyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 25.0/255.0, green: 116.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Pink, whose RBG values are (255, 0, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Turquoise, whose RBG values are (8, 232, 222), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightTurquoiseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 8.0/255.0, green: 232.0/255.0, blue: 222.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bright Ube, whose RBG values are (209, 159, 232), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brightUbeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 209.0/255.0, green: 159.0/255.0, blue: 232.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brilliant Rose (Crayola), whose RBG values are (255, 85, 163), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brilliantRoseCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 85.0/255.0, blue: 163.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brilliant Azure, whose RBG values are (51, 153, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brilliantAzureColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 51.0/255.0, green: 153.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brilliant Lavender, whose RBG values are (244, 187, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brilliantLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 187.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brilliant Rose, whose RBG values are (246, 83, 166), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brilliantRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 246.0/255.0, green: 83.0/255.0, blue: 166.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brink Pink, whose RBG values are (251, 96, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brinkPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 96.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color British Racing Green, whose RBG values are (0, 66, 37), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func britishRacingGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 66.0/255.0, blue: 37.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bronze, whose RBG values are (205, 127, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bronzeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 127.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bronze Yellow, whose RBG values are (115, 112, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bronzeYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 115.0/255.0, green: 112.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown (Crayola), whose RBG values are (180, 103, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brownCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 180.0/255.0, green: 103.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown (Traditional), whose RBG values are (150, 75, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brownTraditionalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 75.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown (Web), whose RBG values are (165, 42, 42), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brownWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 165.0/255.0, green: 42.0/255.0, blue: 42.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 100, whose RBG values are (215, 204, 200), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 204.0/255.0, blue: 200.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 200, whose RBG values are (188, 170, 164), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 188.0/255.0, green: 170.0/255.0, blue: 164.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 300, whose RBG values are (161, 136, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 161.0/255.0, green: 136.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 400, whose RBG values are (141, 110, 99), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 141.0/255.0, green: 110.0/255.0, blue: 99.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 50, whose RBG values are (239, 235, 233), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 235.0/255.0, blue: 233.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 500, whose RBG values are (121, 85, 72), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 121.0/255.0, green: 85.0/255.0, blue: 72.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 600, whose RBG values are (109, 76, 65), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 109.0/255.0, green: 76.0/255.0, blue: 65.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 700, whose RBG values are (93, 64, 55), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 93.0/255.0, green: 64.0/255.0, blue: 55.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 800, whose RBG values are (78, 52, 46), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 78.0/255.0, green: 52.0/255.0, blue: 46.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown 900, whose RBG values are (62, 39, 35), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brown900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 62.0/255.0, green: 39.0/255.0, blue: 35.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown Yellow, whose RBG values are (204, 153, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brownYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 153.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brown-Nose, whose RBG values are (107, 68, 35), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brownNoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 107.0/255.0, green: 68.0/255.0, blue: 35.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Brunswick Green, whose RBG values are (27, 77, 62), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func brunswickGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 27.0/255.0, green: 77.0/255.0, blue: 62.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bubble Gum, whose RBG values are (255, 193, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bubbleGumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 193.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bubbles, whose RBG values are (231, 254, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bubblesColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 231.0/255.0, green: 254.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bud Green, whose RBG values are (123, 182, 97), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func budGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 123.0/255.0, green: 182.0/255.0, blue: 97.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Buff, whose RBG values are (240, 220, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func buffColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 220.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Bulgarian Rose, whose RBG values are (72, 6, 7), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func bulgarianRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 72.0/255.0, green: 6.0/255.0, blue: 7.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Burgundy, whose RBG values are (128, 0, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func burgundyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 0.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Burlywood, whose RBG values are (222, 184, 135), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func burlywoodColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 184.0/255.0, blue: 135.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Burnt Orange (Crayola), whose RBG values are (255, 127, 73), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func burntOrangeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 127.0/255.0, blue: 73.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Burnt Sienna (Crayola), whose RBG values are (234, 126, 93), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func burntSiennaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 234.0/255.0, green: 126.0/255.0, blue: 93.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Burnt Orange, whose RBG values are (204, 85, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func burntOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 85.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Burnt Sienna, whose RBG values are (233, 116, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func burntSiennaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 116.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Burnt Umber, whose RBG values are (138, 51, 36), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func burntUmberColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 138.0/255.0, green: 51.0/255.0, blue: 36.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Byzantine, whose RBG values are (189, 51, 164), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func byzantineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 189.0/255.0, green: 51.0/255.0, blue: 164.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Byzantium, whose RBG values are (112, 41, 99), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func byzantiumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 41.0/255.0, blue: 99.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color CG Blue, whose RBG values are (0, 122, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cGBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 122.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color CG Red, whose RBG values are (224, 60, 49), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cGRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 60.0/255.0, blue: 49.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadet, whose RBG values are (83, 104, 114), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 83.0/255.0, green: 104.0/255.0, blue: 114.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadet Blue (Crayola), whose RBG values are (176, 183, 198), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadetBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 183.0/255.0, blue: 198.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadet Blue, whose RBG values are (95, 158, 160), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadetBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 95.0/255.0, green: 158.0/255.0, blue: 160.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadet Grey, whose RBG values are (145, 163, 176), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadetGreyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 145.0/255.0, green: 163.0/255.0, blue: 176.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadmium Green, whose RBG values are (0, 107, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadmiumGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 107.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadmium Orange, whose RBG values are (237, 135, 45), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadmiumOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 135.0/255.0, blue: 45.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadmium Red, whose RBG values are (227, 0, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadmiumRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 0.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cadmium Yellow, whose RBG values are (255, 246, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cadmiumYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 246.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Café Au Lait, whose RBG values are (166, 123, 91), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func caféAuLaitColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 166.0/255.0, green: 123.0/255.0, blue: 91.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Café Noir, whose RBG values are (75, 54, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func caféNoirColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 75.0/255.0, green: 54.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cal Poly Green, whose RBG values are (30, 77, 43), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func calPolyGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 30.0/255.0, green: 77.0/255.0, blue: 43.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cambridge Blue, whose RBG values are (163, 193, 173), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cambridgeBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 163.0/255.0, green: 193.0/255.0, blue: 173.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Camel, whose RBG values are (193, 154, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func camelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 193.0/255.0, green: 154.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cameo Pink, whose RBG values are (239, 187, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cameoPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 187.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Camouflage Green, whose RBG values are (120, 134, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func camouflageGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 120.0/255.0, green: 134.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Canary (Crayola), whose RBG values are (255, 255, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func canaryCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Canary Yellow, whose RBG values are (255, 239, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func canaryYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 239.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Candy Apple Red, whose RBG values are (255, 8, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func candyAppleRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 8.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Candy Pink, whose RBG values are (228, 113, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func candyPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 113.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Capri, whose RBG values are (0, 191, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func capriColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 191.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Caput Mortuum, whose RBG values are (89, 39, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func caputMortuumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 89.0/255.0, green: 39.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cardinal, whose RBG values are (196, 30, 58), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cardinalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 196.0/255.0, green: 30.0/255.0, blue: 58.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Caribbean Green (Crayola), whose RBG values are (0, 204, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func caribbeanGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 204.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carmine, whose RBG values are (150, 0, 24), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 0.0/255.0, blue: 24.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carmine (M&P), whose RBG values are (215, 0, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carmineMandPColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 0.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carmine Pink, whose RBG values are (235, 76, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carminePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 235.0/255.0, green: 76.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carmine Red, whose RBG values are (255, 0, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carmineRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carnation Pink (Crayola), whose RBG values are (255, 170, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carnationPinkCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 170.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carnation Pink, whose RBG values are (255, 166, 201), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carnationPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 166.0/255.0, blue: 201.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carnelian, whose RBG values are (179, 27, 27), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carnelianColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 27.0/255.0, blue: 27.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carolina Blue, whose RBG values are (86, 160, 211), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carolinaBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 86.0/255.0, green: 160.0/255.0, blue: 211.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Carrot Orange, whose RBG values are (237, 145, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func carrotOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 145.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Castleton Green, whose RBG values are (0, 86, 63), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func castletonGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 86.0/255.0, blue: 63.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Catalina Blue, whose RBG values are (6, 42, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func catalinaBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 6.0/255.0, green: 42.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Catawba, whose RBG values are (112, 54, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func catawbaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 54.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cedar Chest, whose RBG values are (201, 90, 73), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cedarChestColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 201.0/255.0, green: 90.0/255.0, blue: 73.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ceil, whose RBG values are (146, 161, 207), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ceilColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 146.0/255.0, green: 161.0/255.0, blue: 207.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celadon, whose RBG values are (172, 225, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celadonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 172.0/255.0, green: 225.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celadon Blue, whose RBG values are (0, 123, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celadonBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 123.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celadon Green, whose RBG values are (47, 132, 124), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celadonGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 47.0/255.0, green: 132.0/255.0, blue: 124.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celeste, whose RBG values are (178, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celesteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celeste Opaco, whose RBG values are (128, 204, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celesteOpacoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 204.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celeste Pallido, whose RBG values are (204, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celestePallidoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celeste Polvere, whose RBG values are (230, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celestePolvereColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celeste Velato, whose RBG values are (204, 230, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celesteVelatoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 230.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Celestial Blue, whose RBG values are (73, 151, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func celestialBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 73.0/255.0, green: 151.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cerise, whose RBG values are (222, 49, 99), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ceriseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 49.0/255.0, blue: 99.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cerise (Crayola), whose RBG values are (221, 68, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ceriseCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 221.0/255.0, green: 68.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cerise Pink, whose RBG values are (236, 59, 131), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cerisePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 236.0/255.0, green: 59.0/255.0, blue: 131.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cerulean, whose RBG values are (0, 123, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ceruleanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 123.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cerulean (Crayola), whose RBG values are (29, 172, 214), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ceruleanCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 29.0/255.0, green: 172.0/255.0, blue: 214.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cerulean Blue, whose RBG values are (42, 82, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ceruleanBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 42.0/255.0, green: 82.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cerulean Frost, whose RBG values are (109, 155, 195), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ceruleanFrostColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 109.0/255.0, green: 155.0/255.0, blue: 195.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chamoisee, whose RBG values are (160, 120, 90), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chamoiseeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 160.0/255.0, green: 120.0/255.0, blue: 90.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Champagne, whose RBG values are (247, 231, 206), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func champagneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 231.0/255.0, blue: 206.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Charcoal, whose RBG values are (54, 69, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func charcoalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 54.0/255.0, green: 69.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Charleston Green, whose RBG values are (35, 43, 43), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func charlestonGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 35.0/255.0, green: 43.0/255.0, blue: 43.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Charm Pink, whose RBG values are (230, 143, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func charmPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 143.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chartreuse (Traditional), whose RBG values are (223, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chartreuseTraditionalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 223.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chartreuse (Web), whose RBG values are (127, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chartreuseWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 127.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cherry, whose RBG values are (222, 49, 99), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cherryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 49.0/255.0, blue: 99.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cherry Blossom Pink, whose RBG values are (255, 183, 197), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cherryBlossomPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 183.0/255.0, blue: 197.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chestnut, whose RBG values are (149, 69, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chestnutColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 149.0/255.0, green: 69.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chestnut (Crayola), whose RBG values are (188, 93, 88), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chestnutCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 188.0/255.0, green: 93.0/255.0, blue: 88.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color China Pink, whose RBG values are (222, 111, 161), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chinaPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 111.0/255.0, blue: 161.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color China Rose, whose RBG values are (168, 81, 110), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chinaRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 168.0/255.0, green: 81.0/255.0, blue: 110.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chinese Red, whose RBG values are (170, 56, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chineseRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 170.0/255.0, green: 56.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chinese Violet, whose RBG values are (133, 96, 136), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chineseVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 133.0/255.0, green: 96.0/255.0, blue: 136.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chocolate (Traditional), whose RBG values are (123, 63, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chocolateTraditionalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 123.0/255.0, green: 63.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chocolate (Web), whose RBG values are (210, 105, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chocolateWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 210.0/255.0, green: 105.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Chrome Yellow, whose RBG values are (255, 167, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func chromeYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 167.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cinereous, whose RBG values are (152, 129, 123), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cinereousColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 129.0/255.0, blue: 123.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cinnabar, whose RBG values are (227, 66, 52), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cinnabarColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 66.0/255.0, blue: 52.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cinnamon, whose RBG values are (210, 105, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cinnamonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 210.0/255.0, green: 105.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Citrine, whose RBG values are (228, 208, 10), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func citrineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 208.0/255.0, blue: 10.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Citron, whose RBG values are (158, 169, 31), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func citronColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 158.0/255.0, green: 169.0/255.0, blue: 31.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Claret, whose RBG values are (127, 23, 52), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func claretColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 127.0/255.0, green: 23.0/255.0, blue: 52.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Classic Rose, whose RBG values are (251, 204, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func classicRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 204.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cobalt Blue, whose RBG values are (0, 71, 171), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cobaltBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 71.0/255.0, blue: 171.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cocoa Brown, whose RBG values are (210, 105, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cocoaBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 210.0/255.0, green: 105.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Coconut, whose RBG values are (150, 90, 62), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coconutColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 90.0/255.0, blue: 62.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Coffee, whose RBG values are (111, 78, 55), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coffeeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 111.0/255.0, green: 78.0/255.0, blue: 55.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Columbia Blue, whose RBG values are (196, 216, 226), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func columbiaBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 196.0/255.0, green: 216.0/255.0, blue: 226.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Congo Pink, whose RBG values are (248, 131, 121), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func congoPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 131.0/255.0, blue: 121.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cool Black, whose RBG values are (0, 46, 99), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coolBlackColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 46.0/255.0, blue: 99.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cool Grey, whose RBG values are (140, 146, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coolGreyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 140.0/255.0, green: 146.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Copper, whose RBG values are (184, 115, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func copperColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 184.0/255.0, green: 115.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Copper (Crayola Alternate), whose RBG values are (221, 148, 117), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func copperCrayolaAlternateColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 221.0/255.0, green: 148.0/255.0, blue: 117.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Copper (Crayola), whose RBG values are (218, 138, 103), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func copperCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 138.0/255.0, blue: 103.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Copper Penny, whose RBG values are (173, 111, 105), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func copperPennyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 173.0/255.0, green: 111.0/255.0, blue: 105.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Copper Red, whose RBG values are (203, 109, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func copperRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 203.0/255.0, green: 109.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Copper Rose, whose RBG values are (153, 102, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func copperRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 102.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Coquelicot, whose RBG values are (255, 56, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coquelicotColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 56.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Coral, whose RBG values are (255, 127, 80), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coralColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 127.0/255.0, blue: 80.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Coral Pink, whose RBG values are (248, 131, 121), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coralPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 131.0/255.0, blue: 121.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Coral Red, whose RBG values are (255, 64, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coralRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 64.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cordovan, whose RBG values are (137, 63, 69), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cordovanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 137.0/255.0, green: 63.0/255.0, blue: 69.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Corn, whose RBG values are (251, 236, 93), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cornColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 236.0/255.0, blue: 93.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cornell Red, whose RBG values are (179, 27, 27), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cornellRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 27.0/255.0, blue: 27.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cornflower Blue (Crayola), whose RBG values are (154, 206, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cornflowerBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 154.0/255.0, green: 206.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cornflower Blue, whose RBG values are (100, 149, 237), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cornflowerBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 100.0/255.0, green: 149.0/255.0, blue: 237.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cornsilk, whose RBG values are (255, 248, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cornsilkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 248.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cosmic Latte, whose RBG values are (255, 248, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cosmicLatteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 248.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cotton Candy (Crayola), whose RBG values are (255, 188, 217), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cottonCandyCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 188.0/255.0, blue: 217.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Coyote Brown, whose RBG values are (129, 97, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func coyoteBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 129.0/255.0, green: 97.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cream, whose RBG values are (255, 253, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func creamColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 253.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Crimson, whose RBG values are (220, 20, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func crimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 220.0/255.0, green: 20.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Crimson Glory, whose RBG values are (190, 0, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func crimsonGloryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 190.0/255.0, green: 0.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Crimson Red, whose RBG values are (153, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func crimsonRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan (Process), whose RBG values are (0, 183, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanProcessColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 183.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 100, whose RBG values are (178, 235, 242), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 235.0/255.0, blue: 242.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 200, whose RBG values are (128, 222, 234), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 222.0/255.0, blue: 234.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 300, whose RBG values are (77, 208, 225), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 77.0/255.0, green: 208.0/255.0, blue: 225.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 400, whose RBG values are (38, 198, 218), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 38.0/255.0, green: 198.0/255.0, blue: 218.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 50, whose RBG values are (224, 247, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 247.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 500, whose RBG values are (0, 188, 212), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 188.0/255.0, blue: 212.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 600, whose RBG values are (0, 172, 193), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 172.0/255.0, blue: 193.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 700, whose RBG values are (0, 151, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 151.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 800, whose RBG values are (0, 131, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 131.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan 900, whose RBG values are (0, 96, 100), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyan900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 96.0/255.0, blue: 100.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan A100, whose RBG values are (132, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 132.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan A200, whose RBG values are (24, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 24.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan A400, whose RBG values are (0, 229, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 229.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan A700, whose RBG values are (0, 184, 212), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 184.0/255.0, blue: 212.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan Azure, whose RBG values are (78, 130, 180), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanAzureColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 78.0/255.0, green: 130.0/255.0, blue: 180.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan Cobalt Blue, whose RBG values are (40, 88, 156), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanCobaltBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 40.0/255.0, green: 88.0/255.0, blue: 156.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan Cornflower Blue, whose RBG values are (24, 139, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanCornflowerBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 24.0/255.0, green: 139.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyan-Blue Azure, whose RBG values are (70, 130, 191), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyanBlueAzureColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 70.0/255.0, green: 130.0/255.0, blue: 191.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyber Grape, whose RBG values are (88, 66, 124), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyberGrapeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 88.0/255.0, green: 66.0/255.0, blue: 124.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Cyber Yellow, whose RBG values are (255, 211, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func cyberYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 211.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Daffodil, whose RBG values are (255, 255, 49), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func daffodilColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 49.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dandelion, whose RBG values are (240, 225, 48), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dandelionColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 225.0/255.0, blue: 48.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dandelion (Crayola), whose RBG values are (253, 219, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dandelionCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 219.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Blue, whose RBG values are (0, 0, 139), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 0.0/255.0, blue: 139.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Blue-Gray, whose RBG values are (102, 102, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkBlueGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 102.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Brown, whose RBG values are (101, 67, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 101.0/255.0, green: 67.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Brown-Tangelo, whose RBG values are (136, 101, 78), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkBrownTangeloColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 101.0/255.0, blue: 78.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Byzantium, whose RBG values are (93, 57, 84), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkByzantiumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 93.0/255.0, green: 57.0/255.0, blue: 84.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Candy Apple Red, whose RBG values are (164, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkCandyAppleRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 164.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Cerulean, whose RBG values are (8, 69, 126), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkCeruleanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 8.0/255.0, green: 69.0/255.0, blue: 126.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Chestnut, whose RBG values are (152, 105, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkChestnutColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 105.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Coral, whose RBG values are (205, 91, 69), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkCoralColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 91.0/255.0, blue: 69.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Cyan, whose RBG values are (0, 139, 139), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkCyanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 139.0/255.0, blue: 139.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Electric Blue, whose RBG values are (83, 104, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkElectricBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 83.0/255.0, green: 104.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Goldenrod, whose RBG values are (184, 134, 11), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkGoldenrodColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 184.0/255.0, green: 134.0/255.0, blue: 11.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Gray (X11), whose RBG values are (169, 169, 169), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkGrayX11Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 169.0/255.0, green: 169.0/255.0, blue: 169.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Green, whose RBG values are (1, 50, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 1.0/255.0, green: 50.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Green (X11), whose RBG values are (0, 100, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkGreenX11Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 100.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Gunmetal, whose RBG values are (31, 38, 42), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkGunmetalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 31.0/255.0, green: 38.0/255.0, blue: 42.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Imperial Blue 1, whose RBG values are (0, 65, 106), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkImperialBlue1Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 65.0/255.0, blue: 106.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Imperial Blue 2, whose RBG values are (0, 20, 126), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkImperialBlue2Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 20.0/255.0, blue: 126.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Jungle Green, whose RBG values are (26, 36, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkJungleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 26.0/255.0, green: 36.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Khaki, whose RBG values are (189, 183, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkKhakiColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 189.0/255.0, green: 183.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Lava, whose RBG values are (72, 60, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkLavaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 72.0/255.0, green: 60.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Lavender, whose RBG values are (115, 79, 150), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 115.0/255.0, green: 79.0/255.0, blue: 150.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Liver, whose RBG values are (83, 75, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkLiverColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 83.0/255.0, green: 75.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Liver (Horses), whose RBG values are (84, 61, 55), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkLiverHorsesColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 84.0/255.0, green: 61.0/255.0, blue: 55.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Magenta, whose RBG values are (139, 0, 139), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 139.0/255.0, green: 0.0/255.0, blue: 139.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Medium Gray, whose RBG values are (169, 169, 169), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkMediumGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 169.0/255.0, green: 169.0/255.0, blue: 169.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Midnight Blue, whose RBG values are (0, 51, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkMidnightBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 51.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Moss Green, whose RBG values are (74, 93, 35), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkMossGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 74.0/255.0, green: 93.0/255.0, blue: 35.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Olive Green, whose RBG values are (85, 107, 47), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkOliveGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 85.0/255.0, green: 107.0/255.0, blue: 47.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Orange, whose RBG values are (255, 140, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 140.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Orchid, whose RBG values are (153, 50, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkOrchidColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 50.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Pastel Blue, whose RBG values are (119, 158, 203), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPastelBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 119.0/255.0, green: 158.0/255.0, blue: 203.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Pastel Green, whose RBG values are (3, 192, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPastelGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 3.0/255.0, green: 192.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Pastel Purple, whose RBG values are (150, 111, 214), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPastelPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 111.0/255.0, blue: 214.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Pastel Red, whose RBG values are (194, 59, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPastelRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 194.0/255.0, green: 59.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Pink, whose RBG values are (231, 84, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 231.0/255.0, green: 84.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Powder Blue, whose RBG values are (0, 51, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPowderBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 51.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Puce, whose RBG values are (79, 58, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPuceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 79.0/255.0, green: 58.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Purple, whose RBG values are (48, 25, 52), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 48.0/255.0, green: 25.0/255.0, blue: 52.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Raspberry, whose RBG values are (135, 38, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkRaspberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 135.0/255.0, green: 38.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Red, whose RBG values are (139, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 139.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Salmon, whose RBG values are (233, 150, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkSalmonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 150.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Scarlet, whose RBG values are (86, 3, 25), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkScarletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 86.0/255.0, green: 3.0/255.0, blue: 25.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Sea Green, whose RBG values are (143, 188, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkSeaGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 143.0/255.0, green: 188.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Sienna, whose RBG values are (60, 20, 20), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkSiennaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 60.0/255.0, green: 20.0/255.0, blue: 20.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Sky Blue, whose RBG values are (140, 190, 214), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 140.0/255.0, green: 190.0/255.0, blue: 214.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Slate Blue, whose RBG values are (72, 61, 139), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkSlateBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 72.0/255.0, green: 61.0/255.0, blue: 139.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Slate Gray, whose RBG values are (47, 79, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkSlateGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 47.0/255.0, green: 79.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Spring Green, whose RBG values are (23, 114, 69), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkSpringGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 23.0/255.0, green: 114.0/255.0, blue: 69.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Tan, whose RBG values are (145, 129, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkTanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 145.0/255.0, green: 129.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Tangerine, whose RBG values are (255, 168, 18), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkTangerineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 168.0/255.0, blue: 18.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Taupe, whose RBG values are (72, 60, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 72.0/255.0, green: 60.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Terra Cotta, whose RBG values are (204, 78, 92), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkTerraCottaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 78.0/255.0, blue: 92.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Turquoise, whose RBG values are (0, 206, 209), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkTurquoiseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 206.0/255.0, blue: 209.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Vanilla, whose RBG values are (209, 190, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkVanillaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 209.0/255.0, green: 190.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Violet, whose RBG values are (148, 0, 211), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 148.0/255.0, green: 0.0/255.0, blue: 211.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dark Yellow, whose RBG values are (155, 135, 12), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func darkYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 155.0/255.0, green: 135.0/255.0, blue: 12.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dartmouth Green, whose RBG values are (0, 112, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dartmouthGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 112.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Davy's Grey, whose RBG values are (85, 85, 85), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func davysGreyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 85.0/255.0, green: 85.0/255.0, blue: 85.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Debian Red, whose RBG values are (215, 10, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func debianRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 10.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Green, whose RBG values are (5, 102, 8), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 5.0/255.0, green: 102.0/255.0, blue: 8.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 100, whose RBG values are (255, 204, 188), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 204.0/255.0, blue: 188.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 200, whose RBG values are (255, 171, 145), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 171.0/255.0, blue: 145.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 300, whose RBG values are (255, 138, 101), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 138.0/255.0, blue: 101.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 400, whose RBG values are (255, 112, 67), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 112.0/255.0, blue: 67.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 50, whose RBG values are (251, 233, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 233.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 500, whose RBG values are (255, 87, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 87.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 600, whose RBG values are (244, 81, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 81.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 700, whose RBG values are (230, 74, 25), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 74.0/255.0, blue: 25.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 800, whose RBG values are (216, 67, 21), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 216.0/255.0, green: 67.0/255.0, blue: 21.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange 900, whose RBG values are (191, 54, 12), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrange900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 191.0/255.0, green: 54.0/255.0, blue: 12.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange A100, whose RBG values are (255, 158, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrangeA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 158.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange A200, whose RBG values are (255, 110, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrangeA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 110.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange A400, whose RBG values are (255, 61, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrangeA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 61.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Orange A700, whose RBG values are (221, 44, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepOrangeA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 221.0/255.0, green: 44.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 100, whose RBG values are (209, 196, 233), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 209.0/255.0, green: 196.0/255.0, blue: 233.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 200, whose RBG values are (179, 157, 219), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 157.0/255.0, blue: 219.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 300, whose RBG values are (149, 117, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 149.0/255.0, green: 117.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 400, whose RBG values are (126, 87, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 126.0/255.0, green: 87.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 50, whose RBG values are (237, 231, 246), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 231.0/255.0, blue: 246.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 500, whose RBG values are (103, 58, 183), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 103.0/255.0, green: 58.0/255.0, blue: 183.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 600, whose RBG values are (94, 53, 177), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 94.0/255.0, green: 53.0/255.0, blue: 177.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 700, whose RBG values are (81, 45, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 81.0/255.0, green: 45.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 800, whose RBG values are (69, 39, 160), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 69.0/255.0, green: 39.0/255.0, blue: 160.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple 900, whose RBG values are (49, 27, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurple900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 49.0/255.0, green: 27.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple A100, whose RBG values are (179, 136, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurpleA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 136.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple A200, whose RBG values are (124, 77, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurpleA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 77.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple A400, whose RBG values are (101, 31, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurpleA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 101.0/255.0, green: 31.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Purple A700, whose RBG values are (98, 0, 234), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPurpleA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 98.0/255.0, green: 0.0/255.0, blue: 234.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Red, whose RBG values are (133, 1, 1), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 133.0/255.0, green: 1.0/255.0, blue: 1.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Space Sparkle, whose RBG values are (74, 100, 108), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepSpaceSparkleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 74.0/255.0, green: 100.0/255.0, blue: 108.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Taupe, whose RBG values are (126, 94, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 126.0/255.0, green: 94.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Tuscan Red, whose RBG values are (102, 66, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepTuscanRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 66.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Aquamarine, whose RBG values are (64, 130, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepAquamarineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 64.0/255.0, green: 130.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Carmine, whose RBG values are (169, 32, 62), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepCarmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 169.0/255.0, green: 32.0/255.0, blue: 62.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Carmine Pink, whose RBG values are (239, 48, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepCarminePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 48.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Carrot Orange, whose RBG values are (233, 105, 44), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepCarrotOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 105.0/255.0, blue: 44.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Cerise, whose RBG values are (218, 50, 135), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepCeriseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 50.0/255.0, blue: 135.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Champagne, whose RBG values are (250, 214, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepChampagneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 214.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Chestnut, whose RBG values are (185, 78, 72), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepChestnutColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 185.0/255.0, green: 78.0/255.0, blue: 72.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Coffee, whose RBG values are (112, 66, 65), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepCoffeeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 66.0/255.0, blue: 65.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Fuchsia, whose RBG values are (193, 84, 193), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepFuchsiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 193.0/255.0, green: 84.0/255.0, blue: 193.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Green-Cyan Turquoise, whose RBG values are (14, 124, 97), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepGreenCyanTurquoiseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 14.0/255.0, green: 124.0/255.0, blue: 97.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Jungle Green, whose RBG values are (0, 75, 73), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepJungleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 75.0/255.0, blue: 73.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Koamaru, whose RBG values are (51, 51, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepKoamaruColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 51.0/255.0, green: 51.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Lemon, whose RBG values are (245, 199, 26), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepLemonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 199.0/255.0, blue: 26.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Lilac, whose RBG values are (153, 85, 187), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepLilacColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 85.0/255.0, blue: 187.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Magenta, whose RBG values are (204, 0, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 0.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Maroon, whose RBG values are (130, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepMaroonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 130.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Mauve, whose RBG values are (212, 115, 212), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepMauveColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 212.0/255.0, green: 115.0/255.0, blue: 212.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Moss Green, whose RBG values are (53, 94, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepMossGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 53.0/255.0, green: 94.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Peach, whose RBG values are (255, 203, 164), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPeachColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 203.0/255.0, blue: 164.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Pink, whose RBG values are (255, 20, 147), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 20.0/255.0, blue: 147.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Puce, whose RBG values are (169, 92, 104), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepPuceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 169.0/255.0, green: 92.0/255.0, blue: 104.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Ruby, whose RBG values are (132, 63, 91), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepRubyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 132.0/255.0, green: 63.0/255.0, blue: 91.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Saffron, whose RBG values are (255, 153, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepSaffronColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 153.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Sky Blue, whose RBG values are (0, 191, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 191.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Spring Bud, whose RBG values are (85, 107, 47), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepSpringBudColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 85.0/255.0, green: 107.0/255.0, blue: 47.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deep Violet, whose RBG values are (51, 0, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deepVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 51.0/255.0, green: 0.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Deer, whose RBG values are (186, 135, 89), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func deerColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 186.0/255.0, green: 135.0/255.0, blue: 89.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Denim, whose RBG values are (21, 96, 189), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func denimColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 21.0/255.0, green: 96.0/255.0, blue: 189.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Denim (Crayola), whose RBG values are (43, 108, 196), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func denimCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 43.0/255.0, green: 108.0/255.0, blue: 196.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Desaturated Cyan, whose RBG values are (102, 153, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func desaturatedCyanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 153.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Desert, whose RBG values are (193, 154, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func desertColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 193.0/255.0, green: 154.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Desert Sand (Crayola), whose RBG values are (239, 205, 184), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func desertSandCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 205.0/255.0, blue: 184.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Desert Sand, whose RBG values are (237, 201, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func desertSandColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 201.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Desire, whose RBG values are (234, 60, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func desireColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 234.0/255.0, green: 60.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Diamond, whose RBG values are (185, 242, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func diamondColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 185.0/255.0, green: 242.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dim Gray, whose RBG values are (105, 105, 105), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dimGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 105.0/255.0, green: 105.0/255.0, blue: 105.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dirt, whose RBG values are (155, 118, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dirtColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 155.0/255.0, green: 118.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dodger Blue, whose RBG values are (30, 144, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dodgerBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 30.0/255.0, green: 144.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dogwood Rose, whose RBG values are (215, 24, 104), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dogwoodRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 24.0/255.0, blue: 104.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dollar Bill, whose RBG values are (133, 187, 101), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dollarBillColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 133.0/255.0, green: 187.0/255.0, blue: 101.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Donkey Brown, whose RBG values are (102, 76, 40), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func donkeyBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 76.0/255.0, blue: 40.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Drab, whose RBG values are (150, 113, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func drabColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 113.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Duke Blue, whose RBG values are (0, 0, 156), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dukeBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 0.0/255.0, blue: 156.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dust Storm, whose RBG values are (229, 204, 201), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dustStormColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 204.0/255.0, blue: 201.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Dutch White, whose RBG values are (239, 223, 187), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func dutchWhiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 223.0/255.0, blue: 187.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eagle Green, whose RBG values are (0, 73, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func eagleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 73.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Earth Yellow, whose RBG values are (225, 169, 95), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func earthYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 225.0/255.0, green: 169.0/255.0, blue: 95.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ebony, whose RBG values are (85, 93, 80), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ebonyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 85.0/255.0, green: 93.0/255.0, blue: 80.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ecru, whose RBG values are (194, 178, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ecruColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 194.0/255.0, green: 178.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eerie Black, whose RBG values are (27, 27, 27), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func eerieBlackColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 27.0/255.0, green: 27.0/255.0, blue: 27.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eggplant, whose RBG values are (97, 64, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func eggplantColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 97.0/255.0, green: 64.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eggplant (Crayola), whose RBG values are (110, 81, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func eggplantCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 110.0/255.0, green: 81.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eggshell, whose RBG values are (240, 234, 214), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func eggshellColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 234.0/255.0, blue: 214.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Egyptian Blue, whose RBG values are (16, 52, 166), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func egyptianBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 16.0/255.0, green: 52.0/255.0, blue: 166.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Lime (Crayola), whose RBG values are (206, 255, 29), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricLimeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 206.0/255.0, green: 255.0/255.0, blue: 29.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Blue, whose RBG values are (125, 249, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 125.0/255.0, green: 249.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Crimson, whose RBG values are (255, 0, 63), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 63.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Cyan, whose RBG values are (0, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricCyanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Green, whose RBG values are (0, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Indigo, whose RBG values are (111, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricIndigoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 111.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Lavender, whose RBG values are (244, 187, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 187.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Lime, whose RBG values are (204, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricLimeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Purple, whose RBG values are (191, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 191.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Ultramarine, whose RBG values are (63, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricUltramarineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 63.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Violet, whose RBG values are (143, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 143.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Electric Yellow, whose RBG values are (255, 255, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func electricYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Emerald, whose RBG values are (80, 200, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func emeraldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 80.0/255.0, green: 200.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eminence, whose RBG values are (108, 48, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func eminenceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 108.0/255.0, green: 48.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color English Green, whose RBG values are (27, 77, 62), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func englishGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 27.0/255.0, green: 77.0/255.0, blue: 62.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color English Lavender, whose RBG values are (180, 131, 149), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func englishLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 180.0/255.0, green: 131.0/255.0, blue: 149.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color English Red, whose RBG values are (171, 75, 82), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func englishRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 171.0/255.0, green: 75.0/255.0, blue: 82.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color English Violet, whose RBG values are (86, 60, 92), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func englishVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 86.0/255.0, green: 60.0/255.0, blue: 92.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eton Blue, whose RBG values are (150, 200, 162), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func etonBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 200.0/255.0, blue: 162.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Eucalyptus, whose RBG values are (68, 215, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func eucalyptusColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 68.0/255.0, green: 215.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fallow, whose RBG values are (193, 154, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fallowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 193.0/255.0, green: 154.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Falu Red, whose RBG values are (128, 24, 24), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func faluRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 24.0/255.0, blue: 24.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fandango, whose RBG values are (181, 51, 137), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fandangoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 181.0/255.0, green: 51.0/255.0, blue: 137.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fandango Pink, whose RBG values are (222, 82, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fandangoPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 82.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fashion Fuchsia, whose RBG values are (244, 0, 161), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fashionFuchsiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 0.0/255.0, blue: 161.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fawn, whose RBG values are (229, 170, 112), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fawnColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 170.0/255.0, blue: 112.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Feldgrau, whose RBG values are (77, 93, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func feldgrauColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 77.0/255.0, green: 93.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Feldspar, whose RBG values are (253, 213, 177), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func feldsparColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 213.0/255.0, blue: 177.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fern (Crayola), whose RBG values are (113, 188, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fernCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 113.0/255.0, green: 188.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fern Green, whose RBG values are (79, 121, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fernGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 79.0/255.0, green: 121.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ferrari Red, whose RBG values are (255, 40, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ferrariRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 40.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Field Drab, whose RBG values are (108, 84, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fieldDrabColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 108.0/255.0, green: 84.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fire Engine Red, whose RBG values are (206, 32, 41), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fireEngineRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 206.0/255.0, green: 32.0/255.0, blue: 41.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Firebrick, whose RBG values are (178, 34, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func firebrickColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 34.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Flame, whose RBG values are (226, 88, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func flameColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 226.0/255.0, green: 88.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Flamingo Pink, whose RBG values are (252, 142, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func flamingoPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 142.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Flattery, whose RBG values are (107, 68, 35), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func flatteryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 107.0/255.0, green: 68.0/255.0, blue: 35.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Flavescent, whose RBG values are (247, 233, 142), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func flavescentColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 233.0/255.0, blue: 142.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Flax, whose RBG values are (238, 220, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func flaxColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 220.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Flirt, whose RBG values are (162, 0, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func flirtColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 162.0/255.0, green: 0.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Floral White, whose RBG values are (255, 250, 240), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func floralWhiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 250.0/255.0, blue: 240.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fluorescent Orange, whose RBG values are (255, 191, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fluorescentOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 191.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fluorescent Pink, whose RBG values are (255, 20, 147), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fluorescentPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 20.0/255.0, blue: 147.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fluorescent Yellow, whose RBG values are (204, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fluorescentYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Folly, whose RBG values are (255, 0, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func follyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Forest Green (Crayola), whose RBG values are (109, 174, 129), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func forestGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 109.0/255.0, green: 174.0/255.0, blue: 129.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Forest Green (Traditional), whose RBG values are (1, 68, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func forestGreenTraditionalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 1.0/255.0, green: 68.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Forest Green (Web), whose RBG values are (34, 139, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func forestGreenWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 34.0/255.0, green: 139.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Beige, whose RBG values are (166, 123, 91), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchBeigeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 166.0/255.0, green: 123.0/255.0, blue: 91.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Bistre, whose RBG values are (133, 109, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchBistreColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 133.0/255.0, green: 109.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Blue, whose RBG values are (0, 114, 187), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 114.0/255.0, blue: 187.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Fuchsia, whose RBG values are (253, 63, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchFuchsiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 63.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Lilac, whose RBG values are (134, 96, 142), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchLilacColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 134.0/255.0, green: 96.0/255.0, blue: 142.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Lime, whose RBG values are (158, 253, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchLimeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 158.0/255.0, green: 253.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Mauve, whose RBG values are (212, 115, 212), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchMauveColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 212.0/255.0, green: 115.0/255.0, blue: 212.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Pink, whose RBG values are (253, 108, 158), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 108.0/255.0, blue: 158.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Plum, whose RBG values are (129, 20, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchPlumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 129.0/255.0, green: 20.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Puce, whose RBG values are (78, 22, 9), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchPuceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 78.0/255.0, green: 22.0/255.0, blue: 9.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Raspberry, whose RBG values are (199, 44, 72), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchRaspberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 199.0/255.0, green: 44.0/255.0, blue: 72.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Rose, whose RBG values are (246, 74, 138), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 246.0/255.0, green: 74.0/255.0, blue: 138.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Sky Blue, whose RBG values are (119, 181, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 119.0/255.0, green: 181.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Violet, whose RBG values are (136, 6, 206), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 6.0/255.0, blue: 206.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color French Wine, whose RBG values are (172, 30, 68), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func frenchWineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 172.0/255.0, green: 30.0/255.0, blue: 68.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fresh Air, whose RBG values are (166, 231, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func freshAirColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 166.0/255.0, green: 231.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fuchsia, whose RBG values are (255, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fuchsiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fuchsia (Crayola), whose RBG values are (195, 100, 197), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fuchsiaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 195.0/255.0, green: 100.0/255.0, blue: 197.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fuchsia Pink, whose RBG values are (255, 119, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fuchsiaPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 119.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fuchsia Purple, whose RBG values are (204, 57, 123), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fuchsiaPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 57.0/255.0, blue: 123.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fuchsia Rose, whose RBG values are (199, 67, 117), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fuchsiaRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 199.0/255.0, green: 67.0/255.0, blue: 117.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fulvous, whose RBG values are (228, 132, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fulvousColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 132.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Fuzzy Wuzzy (Crayola), whose RBG values are (204, 102, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func fuzzyWuzzyCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 102.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color GO Green, whose RBG values are (0, 171, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func gOGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 171.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gainsboro, whose RBG values are (220, 220, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func gainsboroColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 220.0/255.0, green: 220.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gamboge, whose RBG values are (228, 155, 15), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func gambogeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 155.0/255.0, blue: 15.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gamboge Orange (Brown), whose RBG values are (152, 102, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func gambogeOrangeBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 102.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Generic Viridian, whose RBG values are (0, 127, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func genericViridianColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 127.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ghost White, whose RBG values are (248, 248, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ghostWhiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 248.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Giants Orange, whose RBG values are (254, 90, 29), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func giantsOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 90.0/255.0, blue: 29.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Glaucous, whose RBG values are (96, 130, 182), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func glaucousColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 96.0/255.0, green: 130.0/255.0, blue: 182.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Glitter, whose RBG values are (230, 232, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func glitterColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 232.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gold (Crayola), whose RBG values are (231, 198, 151), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 231.0/255.0, green: 198.0/255.0, blue: 151.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gold (Metallic), whose RBG values are (212, 175, 55), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldMetallicColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 212.0/255.0, green: 175.0/255.0, blue: 55.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gold (Web) (Golden), whose RBG values are (255, 215, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldWebGoldenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 215.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gold Fusion, whose RBG values are (133, 117, 78), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldFusionColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 133.0/255.0, green: 117.0/255.0, blue: 78.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Golden Brown, whose RBG values are (153, 101, 21), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldenBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 101.0/255.0, blue: 21.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Golden Poppy, whose RBG values are (252, 194, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldenPoppyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 194.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Golden Yellow, whose RBG values are (255, 223, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldenYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 223.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Goldenrod, whose RBG values are (218, 165, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldenrodColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 165.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Goldenrod (Crayola), whose RBG values are (252, 217, 117), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func goldenrodCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 217.0/255.0, blue: 117.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Granny Smith Apple (Crayola), whose RBG values are (168, 228, 160), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grannySmithAppleCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 168.0/255.0, green: 228.0/255.0, blue: 160.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grape, whose RBG values are (111, 45, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grapeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 111.0/255.0, green: 45.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gray (Alternate), whose RBG values are (128, 128, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grayAlternateColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 128.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gray (Crayola), whose RBG values are (149, 145, 140), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grayCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 149.0/255.0, green: 145.0/255.0, blue: 140.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gray (HTML/CSS Gray), whose RBG values are (128, 128, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grayHTMLCSSGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 128.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gray (X11 Gray), whose RBG values are (190, 190, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grayX11GrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 190.0/255.0, green: 190.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gray-Asparagus, whose RBG values are (70, 89, 69), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grayAsparagusColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 70.0/255.0, green: 89.0/255.0, blue: 69.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gray-Blue, whose RBG values are (140, 146, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grayBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 140.0/255.0, green: 146.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (Color Wheel) (X11 Green), whose RBG values are (0, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenColorWheelX11GreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (Crayola), whose RBG values are (28, 172, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 28.0/255.0, green: 172.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (HTML/CSS Color), whose RBG values are (0, 128, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenHTMLCSSColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 128.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (Munsell), whose RBG values are (0, 168, 119), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenMunsellColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 168.0/255.0, blue: 119.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (NCS), whose RBG values are (0, 159, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenNCSColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 159.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (Pantone), whose RBG values are (0, 173, 67), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenPantoneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 173.0/255.0, blue: 67.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (Pigment), whose RBG values are (0, 165, 80), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenPigmentColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 165.0/255.0, blue: 80.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green (RYB), whose RBG values are (102, 176, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenRYBColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 176.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 100, whose RBG values are (200, 230, 201), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 200.0/255.0, green: 230.0/255.0, blue: 201.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 200, whose RBG values are (165, 214, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 165.0/255.0, green: 214.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 300, whose RBG values are (129, 199, 132), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 129.0/255.0, green: 199.0/255.0, blue: 132.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 400, whose RBG values are (102, 187, 106), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 187.0/255.0, blue: 106.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 50, whose RBG values are (232, 245, 233), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 232.0/255.0, green: 245.0/255.0, blue: 233.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 500, whose RBG values are (76, 175, 80), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 76.0/255.0, green: 175.0/255.0, blue: 80.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 600, whose RBG values are (67, 160, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 67.0/255.0, green: 160.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 700, whose RBG values are (56, 142, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 56.0/255.0, green: 142.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 800, whose RBG values are (46, 125, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 46.0/255.0, green: 125.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green 900, whose RBG values are (27, 94, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func green900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 27.0/255.0, green: 94.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green A100, whose RBG values are (185, 246, 202), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 185.0/255.0, green: 246.0/255.0, blue: 202.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green A200, whose RBG values are (105, 240, 174), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 105.0/255.0, green: 240.0/255.0, blue: 174.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green A400, whose RBG values are (0, 230, 118), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 230.0/255.0, blue: 118.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green A700, whose RBG values are (0, 200, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 200.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green Blue (Crayola), whose RBG values are (17, 100, 180), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 17.0/255.0, green: 100.0/255.0, blue: 180.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green Yellow (Crayola), whose RBG values are (240, 232, 145), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenYellowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 232.0/255.0, blue: 145.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green Yellow, whose RBG values are (173, 255, 47), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 173.0/255.0, green: 255.0/255.0, blue: 47.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Green-Cyan, whose RBG values are (0, 153, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func greenCyanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 153.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 100, whose RBG values are (245, 245, 245), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 245.0/255.0, blue: 245.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 200, whose RBG values are (238, 238, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 238.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 300, whose RBG values are (224, 224, 224), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 224.0/255.0, blue: 224.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 400, whose RBG values are (189, 189, 189), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 189.0/255.0, green: 189.0/255.0, blue: 189.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 50, whose RBG values are (250, 250, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 250.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 500, whose RBG values are (158, 158, 158), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 158.0/255.0, green: 158.0/255.0, blue: 158.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 600, whose RBG values are (117, 117, 117), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 117.0/255.0, green: 117.0/255.0, blue: 117.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 700, whose RBG values are (97, 97, 97), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 97.0/255.0, green: 97.0/255.0, blue: 97.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 800, whose RBG values are (66, 66, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 66.0/255.0, green: 66.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grey 900, whose RBG values are (33, 33, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grey900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 33.0/255.0, green: 33.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grizzly, whose RBG values are (136, 88, 24), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grizzlyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 88.0/255.0, blue: 24.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grullo, whose RBG values are (169, 154, 134), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grulloColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 169.0/255.0, green: 154.0/255.0, blue: 134.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Grussrel, whose RBG values are (176, 101, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func grussrelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 101.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Gunmetal, whose RBG values are (42, 52, 57), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func gunmetalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 42.0/255.0, green: 52.0/255.0, blue: 57.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Guppie Green, whose RBG values are (0, 255, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func guppieGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Halayà úbe, whose RBG values are (102, 55, 84), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func halayàúbeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 55.0/255.0, blue: 84.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Han Blue, whose RBG values are (68, 108, 207), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hanBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 68.0/255.0, green: 108.0/255.0, blue: 207.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Han Purple, whose RBG values are (82, 24, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hanPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 82.0/255.0, green: 24.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Hansa Yellow, whose RBG values are (233, 214, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hansaYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 214.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Harlequin, whose RBG values are (63, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func harlequinColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 63.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Harlequin Green, whose RBG values are (70, 203, 24), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func harlequinGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 70.0/255.0, green: 203.0/255.0, blue: 24.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Harvard Crimson, whose RBG values are (201, 0, 22), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func harvardCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 201.0/255.0, green: 0.0/255.0, blue: 22.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Harvest Gold, whose RBG values are (218, 145, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func harvestGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 145.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Heart Gold, whose RBG values are (128, 128, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func heartGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 128.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Heliotrope, whose RBG values are (223, 115, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func heliotropeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 223.0/255.0, green: 115.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Heliotrope Gray, whose RBG values are (170, 152, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func heliotropeGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 170.0/255.0, green: 152.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Heliotrope Magenta, whose RBG values are (170, 0, 187), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func heliotropeMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 170.0/255.0, green: 0.0/255.0, blue: 187.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Hollywood Cerise, whose RBG values are (244, 0, 161), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hollywoodCeriseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 0.0/255.0, blue: 161.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Honeydew, whose RBG values are (240, 255, 240), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func honeydewColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 255.0/255.0, blue: 240.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Honolulu Blue, whose RBG values are (0, 109, 176), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func honoluluBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 109.0/255.0, blue: 176.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Hooker's Green, whose RBG values are (73, 121, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hookersGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 73.0/255.0, green: 121.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Hot Magenta (Crayola), whose RBG values are (255, 29, 206), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hotMagentaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 29.0/255.0, blue: 206.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Hot Pink, whose RBG values are (255, 105, 180), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hotPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 105.0/255.0, blue: 180.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Hunter Green, whose RBG values are (53, 94, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func hunterGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 53.0/255.0, green: 94.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Iceberg, whose RBG values are (113, 166, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func icebergColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 113.0/255.0, green: 166.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Icterine, whose RBG values are (252, 247, 94), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func icterineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 247.0/255.0, blue: 94.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Illuminating Emerald, whose RBG values are (49, 145, 119), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func illuminatingEmeraldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 49.0/255.0, green: 145.0/255.0, blue: 119.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Imperial, whose RBG values are (96, 47, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func imperialColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 96.0/255.0, green: 47.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Imperial Blue, whose RBG values are (0, 35, 149), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func imperialBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 35.0/255.0, blue: 149.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Imperial Purple, whose RBG values are (102, 2, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func imperialPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 2.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Imperial Red, whose RBG values are (237, 41, 57), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func imperialRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 41.0/255.0, blue: 57.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Inchworm (Crayola), whose RBG values are (178, 236, 93), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func inchwormCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 236.0/255.0, blue: 93.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Independence, whose RBG values are (76, 81, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func independenceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 76.0/255.0, green: 81.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color India Green, whose RBG values are (19, 136, 8), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indiaGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 19.0/255.0, green: 136.0/255.0, blue: 8.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indian Red, whose RBG values are (205, 92, 92), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indianRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 92.0/255.0, blue: 92.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indian Yellow, whose RBG values are (227, 168, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indianYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 168.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo, whose RBG values are (75, 0, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 75.0/255.0, green: 0.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo (Crayola), whose RBG values are (93, 118, 203), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigoCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 93.0/255.0, green: 118.0/255.0, blue: 203.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 100, whose RBG values are (197, 202, 233), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 197.0/255.0, green: 202.0/255.0, blue: 233.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 200, whose RBG values are (159, 168, 218), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 159.0/255.0, green: 168.0/255.0, blue: 218.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 300, whose RBG values are (121, 134, 203), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 121.0/255.0, green: 134.0/255.0, blue: 203.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 400, whose RBG values are (92, 107, 192), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 92.0/255.0, green: 107.0/255.0, blue: 192.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 50, whose RBG values are (232, 234, 246), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 232.0/255.0, green: 234.0/255.0, blue: 246.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 500, whose RBG values are (63, 81, 181), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 63.0/255.0, green: 81.0/255.0, blue: 181.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 600, whose RBG values are (57, 73, 171), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 57.0/255.0, green: 73.0/255.0, blue: 171.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 700, whose RBG values are (48, 63, 159), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 48.0/255.0, green: 63.0/255.0, blue: 159.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 800, whose RBG values are (40, 53, 147), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 40.0/255.0, green: 53.0/255.0, blue: 147.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo 900, whose RBG values are (26, 35, 126), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigo900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 26.0/255.0, green: 35.0/255.0, blue: 126.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo A100, whose RBG values are (140, 158, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigoA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 140.0/255.0, green: 158.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo A200, whose RBG values are (83, 109, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigoA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 83.0/255.0, green: 109.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo A400, whose RBG values are (61, 90, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigoA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 61.0/255.0, green: 90.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo A700, whose RBG values are (48, 79, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigoA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 48.0/255.0, green: 79.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Indigo Dye, whose RBG values are (9, 31, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func indigoDyeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 9.0/255.0, green: 31.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color International Klein Blue, whose RBG values are (0, 47, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func internationalKleinBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 47.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color International Orange (Aerospace), whose RBG values are (255, 79, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func internationalOrangeAerospaceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 79.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color International Orange (Engineering), whose RBG values are (186, 22, 12), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func internationalOrangeEngineeringColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 186.0/255.0, green: 22.0/255.0, blue: 12.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color International Orange (Golden Gate Bridge), whose RBG values are (192, 54, 44), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func internationalOrangeGoldenGateBridgeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 192.0/255.0, green: 54.0/255.0, blue: 44.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Iris, whose RBG values are (90, 79, 207), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func irisColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 90.0/255.0, green: 79.0/255.0, blue: 207.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Irresistible, whose RBG values are (179, 68, 108), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func irresistibleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 68.0/255.0, blue: 108.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Isabelline, whose RBG values are (244, 240, 236), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func isabellineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 240.0/255.0, blue: 236.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Islamic Green, whose RBG values are (0, 144, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func islamicGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 144.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Italian Sky Blue, whose RBG values are (178, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func italianSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ivory, whose RBG values are (255, 255, 240), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ivoryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 240.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jade, whose RBG values are (0, 168, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jadeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 168.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Japanese Carmine, whose RBG values are (157, 41, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func japaneseCarmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 157.0/255.0, green: 41.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Japanese Indigo, whose RBG values are (38, 67, 72), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func japaneseIndigoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 38.0/255.0, green: 67.0/255.0, blue: 72.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Japanese Violet, whose RBG values are (91, 50, 86), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func japaneseVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 91.0/255.0, green: 50.0/255.0, blue: 86.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jasmine, whose RBG values are (248, 222, 126), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jasmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 222.0/255.0, blue: 126.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jasper, whose RBG values are (215, 59, 62), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jasperColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 59.0/255.0, blue: 62.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jawad/Chicken Color (HTML/CSS) (Khaki), whose RBG values are (195, 176, 145), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jawadChickenColorHTMLCSSKhakiColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 195.0/255.0, green: 176.0/255.0, blue: 145.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jazzberry Jam (Crayola), whose RBG values are (202, 55, 103), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jazzberryJamCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 202.0/255.0, green: 55.0/255.0, blue: 103.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jelly Bean, whose RBG values are (218, 97, 78), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jellyBeanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 97.0/255.0, blue: 78.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jet, whose RBG values are (52, 52, 52), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 52.0/255.0, green: 52.0/255.0, blue: 52.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jonquil, whose RBG values are (244, 202, 22), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jonquilColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 202.0/255.0, blue: 22.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jordy Blue, whose RBG values are (138, 185, 241), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jordyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 138.0/255.0, green: 185.0/255.0, blue: 241.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color June Bud, whose RBG values are (189, 218, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func juneBudColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 189.0/255.0, green: 218.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Jungle Green (Crayola), whose RBG values are (59, 176, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func jungleGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 59.0/255.0, green: 176.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color KU Crimson, whose RBG values are (232, 0, 13), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func kUCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 232.0/255.0, green: 0.0/255.0, blue: 13.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Kelly Green, whose RBG values are (76, 187, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func kellyGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 76.0/255.0, green: 187.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Kenyan Copper, whose RBG values are (124, 28, 5), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func kenyanCopperColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 28.0/255.0, blue: 5.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Keppel, whose RBG values are (58, 176, 158), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func keppelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 58.0/255.0, green: 176.0/255.0, blue: 158.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Khaki, whose RBG values are (189, 183, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func khakiColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 189.0/255.0, green: 183.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Khaki (X11) (Light Khaki), whose RBG values are (240, 230, 140), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func khakiX11LightKhakiColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 230.0/255.0, blue: 140.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Kobe, whose RBG values are (136, 45, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func kobeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 45.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Kobi, whose RBG values are (231, 159, 196), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func kobiColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 231.0/255.0, green: 159.0/255.0, blue: 196.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Kombu Green, whose RBG values are (53, 66, 48), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func kombuGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 53.0/255.0, green: 66.0/255.0, blue: 48.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color La Salle Green, whose RBG values are (8, 120, 48), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func laSalleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 8.0/255.0, green: 120.0/255.0, blue: 48.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Languid Lavender, whose RBG values are (214, 202, 221), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func languidLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 214.0/255.0, green: 202.0/255.0, blue: 221.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lapis Lazuli, whose RBG values are (38, 97, 156), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lapisLazuliColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 38.0/255.0, green: 97.0/255.0, blue: 156.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Laser Lemon (Crayola), whose RBG values are (254, 254, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func laserLemonCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 254.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Laurel Green, whose RBG values are (169, 186, 157), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func laurelGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 169.0/255.0, green: 186.0/255.0, blue: 157.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lava, whose RBG values are (207, 16, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 207.0/255.0, green: 16.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender (Crayola), whose RBG values are (252, 180, 213), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 180.0/255.0, blue: 213.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender (Floral), whose RBG values are (181, 126, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderFloralColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 181.0/255.0, green: 126.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender (Web), whose RBG values are (230, 230, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 230.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Blue, whose RBG values are (204, 204, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 204.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Blush, whose RBG values are (255, 240, 245), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderBlushColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 240.0/255.0, blue: 245.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Gray, whose RBG values are (196, 195, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 196.0/255.0, green: 195.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Indigo, whose RBG values are (148, 87, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderIndigoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 148.0/255.0, green: 87.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Magenta, whose RBG values are (238, 130, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 130.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Mist, whose RBG values are (230, 230, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderMistColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 230.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Pink, whose RBG values are (251, 174, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 174.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Purple, whose RBG values are (150, 123, 182), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 123.0/255.0, blue: 182.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lavender Rose, whose RBG values are (251, 160, 227), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lavenderRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 160.0/255.0, blue: 227.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lawn Green, whose RBG values are (124, 252, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lawnGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 252.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lemon, whose RBG values are (255, 247, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lemonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 247.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lemon Yellow (Crayola), whose RBG values are (255, 244, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lemonYellowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 244.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lemon Chiffon, whose RBG values are (255, 250, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lemonChiffonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 250.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lemon Curry, whose RBG values are (204, 160, 29), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lemonCurryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 160.0/255.0, blue: 29.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lemon Glacier, whose RBG values are (253, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lemonGlacierColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lemon Lime, whose RBG values are (227, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lemonLimeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lemon Meringue, whose RBG values are (246, 234, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lemonMeringueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 246.0/255.0, green: 234.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lenurple, whose RBG values are (186, 147, 216), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lenurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 186.0/255.0, green: 147.0/255.0, blue: 216.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Liberty, whose RBG values are (84, 90, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func libertyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 84.0/255.0, green: 90.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Licorice, whose RBG values are (26, 17, 16), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func licoriceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 26.0/255.0, green: 17.0/255.0, blue: 16.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue (Crayola), whose RBG values are (173, 216, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 173.0/255.0, green: 216.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 100, whose RBG values are (179, 229, 252), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 229.0/255.0, blue: 252.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 200, whose RBG values are (129, 212, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 129.0/255.0, green: 212.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 300, whose RBG values are (79, 195, 247), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 79.0/255.0, green: 195.0/255.0, blue: 247.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 400, whose RBG values are (41, 182, 246), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 41.0/255.0, green: 182.0/255.0, blue: 246.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 50, whose RBG values are (225, 245, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 225.0/255.0, green: 245.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 500, whose RBG values are (3, 169, 244), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 3.0/255.0, green: 169.0/255.0, blue: 244.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 600, whose RBG values are (3, 155, 229), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 3.0/255.0, green: 155.0/255.0, blue: 229.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 700, whose RBG values are (2, 136, 209), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 2.0/255.0, green: 136.0/255.0, blue: 209.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 800, whose RBG values are (2, 119, 189), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 2.0/255.0, green: 119.0/255.0, blue: 189.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue 900, whose RBG values are (1, 87, 155), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlue900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 1.0/255.0, green: 87.0/255.0, blue: 155.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue A100, whose RBG values are (128, 216, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlueA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 216.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue A200, whose RBG values are (64, 196, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlueA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 64.0/255.0, green: 196.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue A400, whose RBG values are (0, 176, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlueA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 176.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue A700, whose RBG values are (0, 145, 234), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlueA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 145.0/255.0, blue: 234.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light French Beige, whose RBG values are (200, 173, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightFrenchBeigeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 200.0/255.0, green: 173.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 100, whose RBG values are (220, 237, 200), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 220.0/255.0, green: 237.0/255.0, blue: 200.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 200, whose RBG values are (197, 225, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 197.0/255.0, green: 225.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 300, whose RBG values are (174, 213, 129), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 174.0/255.0, green: 213.0/255.0, blue: 129.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 400, whose RBG values are (156, 204, 101), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 156.0/255.0, green: 204.0/255.0, blue: 101.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 50, whose RBG values are (241, 248, 233), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 241.0/255.0, green: 248.0/255.0, blue: 233.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 500, whose RBG values are (139, 195, 74), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 139.0/255.0, green: 195.0/255.0, blue: 74.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 600, whose RBG values are (124, 179, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 179.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 700, whose RBG values are (104, 159, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 104.0/255.0, green: 159.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 800, whose RBG values are (85, 139, 47), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 85.0/255.0, green: 139.0/255.0, blue: 47.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green 900, whose RBG values are (51, 105, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreen900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 51.0/255.0, green: 105.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green A100, whose RBG values are (204, 255, 144), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreenA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 255.0/255.0, blue: 144.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green A200, whose RBG values are (178, 255, 89), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreenA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 255.0/255.0, blue: 89.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green A400, whose RBG values are (118, 255, 3), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreenA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 118.0/255.0, green: 255.0/255.0, blue: 3.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green A700, whose RBG values are (100, 221, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreenA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 100.0/255.0, green: 221.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Thulian Pink, whose RBG values are (230, 143, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightThulianPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 143.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Apricot, whose RBG values are (253, 213, 177), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightApricotColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 213.0/255.0, blue: 177.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Blue, whose RBG values are (173, 216, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 173.0/255.0, green: 216.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Brilliant Red, whose RBG values are (254, 46, 46), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBrilliantRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 46.0/255.0, blue: 46.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Brown, whose RBG values are (181, 101, 29), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 181.0/255.0, green: 101.0/255.0, blue: 29.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Carmine Pink, whose RBG values are (230, 103, 113), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightCarminePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 103.0/255.0, blue: 113.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Cobalt Blue, whose RBG values are (136, 172, 224), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightCobaltBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 172.0/255.0, blue: 224.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Coral, whose RBG values are (240, 128, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightCoralColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 128.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Cornflower Blue, whose RBG values are (147, 204, 234), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightCornflowerBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 147.0/255.0, green: 204.0/255.0, blue: 234.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Crimson, whose RBG values are (245, 105, 145), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 105.0/255.0, blue: 145.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Cyan, whose RBG values are (224, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightCyanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Deep Pink, whose RBG values are (255, 92, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightDeepPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 92.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Fuchsia Pink, whose RBG values are (249, 132, 239), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightFuchsiaPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 249.0/255.0, green: 132.0/255.0, blue: 239.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Goldenrod Yellow, whose RBG values are (250, 250, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGoldenrodYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 250.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Gray (Alternate), whose RBG values are (211, 211, 211), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGrayAlternateColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 211.0/255.0, green: 211.0/255.0, blue: 211.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Grayish Magenta, whose RBG values are (204, 153, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGrayishMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 153.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Green, whose RBG values are (144, 238, 144), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 144.0/255.0, green: 238.0/255.0, blue: 144.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Hot Pink, whose RBG values are (255, 179, 222), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightHotPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 179.0/255.0, blue: 222.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Khaki, whose RBG values are (240, 230, 140), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightKhakiColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 230.0/255.0, blue: 140.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Medium Orchid, whose RBG values are (211, 155, 203), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightMediumOrchidColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 211.0/255.0, green: 155.0/255.0, blue: 203.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Moss Green, whose RBG values are (173, 223, 173), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightMossGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 173.0/255.0, green: 223.0/255.0, blue: 173.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Orchid, whose RBG values are (230, 168, 215), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightOrchidColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 168.0/255.0, blue: 215.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Pastel Purple, whose RBG values are (177, 156, 217), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightPastelPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 177.0/255.0, green: 156.0/255.0, blue: 217.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Pink, whose RBG values are (255, 182, 193), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 182.0/255.0, blue: 193.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Red Ochre, whose RBG values are (233, 116, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightRedOchreColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 116.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Salmon, whose RBG values are (255, 160, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightSalmonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 160.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Salmon Pink, whose RBG values are (255, 153, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightSalmonPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 153.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Sea Green, whose RBG values are (32, 178, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightSeaGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 32.0/255.0, green: 178.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Sky Blue, whose RBG values are (135, 206, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 135.0/255.0, green: 206.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Slate Gray, whose RBG values are (119, 136, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightSlateGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 119.0/255.0, green: 136.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Steel Blue, whose RBG values are (176, 196, 222), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightSteelBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 196.0/255.0, blue: 222.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Taupe, whose RBG values are (179, 139, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 139.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Light Yellow, whose RBG values are (255, 255, 224), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lightYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 224.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lilac, whose RBG values are (200, 162, 200), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lilacColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 200.0/255.0, green: 162.0/255.0, blue: 200.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime (Color Wheel), whose RBG values are (191, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limeColorWheelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 191.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime (Web) (X11 Green), whose RBG values are (0, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limeWebX11GreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 100, whose RBG values are (240, 244, 195), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 244.0/255.0, blue: 195.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 200, whose RBG values are (230, 238, 156), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 238.0/255.0, blue: 156.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 300, whose RBG values are (220, 231, 117), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 220.0/255.0, green: 231.0/255.0, blue: 117.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 400, whose RBG values are (212, 225, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 212.0/255.0, green: 225.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 50, whose RBG values are (249, 251, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 249.0/255.0, green: 251.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 500, whose RBG values are (205, 220, 57), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 220.0/255.0, blue: 57.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 600, whose RBG values are (192, 202, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 192.0/255.0, green: 202.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 700, whose RBG values are (175, 180, 43), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 175.0/255.0, green: 180.0/255.0, blue: 43.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 800, whose RBG values are (158, 157, 36), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 158.0/255.0, green: 157.0/255.0, blue: 36.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime 900, whose RBG values are (130, 119, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lime900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 130.0/255.0, green: 119.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime A100, whose RBG values are (244, 255, 129), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limeA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 255.0/255.0, blue: 129.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime A200, whose RBG values are (238, 255, 65), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limeA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 255.0/255.0, blue: 65.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime A400, whose RBG values are (198, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limeA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 198.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime A700, whose RBG values are (174, 234, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limeA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 174.0/255.0, green: 234.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lime Green, whose RBG values are (50, 205, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limeGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 50.0/255.0, green: 205.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Limerick, whose RBG values are (157, 194, 9), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func limerickColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 157.0/255.0, green: 194.0/255.0, blue: 9.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lincoln Green, whose RBG values are (25, 89, 5), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lincolnGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 25.0/255.0, green: 89.0/255.0, blue: 5.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Linen, whose RBG values are (250, 240, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func linenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 240.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lion, whose RBG values are (193, 154, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lionColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 193.0/255.0, green: 154.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Liseran Purple, whose RBG values are (222, 111, 161), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func liseranPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 111.0/255.0, blue: 161.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Little Boy Blue, whose RBG values are (108, 160, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func littleBoyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 108.0/255.0, green: 160.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Liver, whose RBG values are (103, 76, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func liverColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 103.0/255.0, green: 76.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Liver (Dogs), whose RBG values are (184, 109, 41), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func liverDogsColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 184.0/255.0, green: 109.0/255.0, blue: 41.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Liver (Organ), whose RBG values are (108, 46, 31), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func liverOrganColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 108.0/255.0, green: 46.0/255.0, blue: 31.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Liver Chestnut, whose RBG values are (152, 116, 86), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func liverChestnutColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 116.0/255.0, blue: 86.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Livid, whose RBG values are (102, 153, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lividColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 153.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lumber, whose RBG values are (255, 228, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lumberColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 228.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Lust, whose RBG values are (230, 32, 32), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func lustColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 32.0/255.0, blue: 32.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color MSU Green, whose RBG values are (24, 69, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mSUGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 24.0/255.0, green: 69.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Macaroni And Cheese (Crayola), whose RBG values are (255, 189, 136), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func macaroniAndCheeseCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 189.0/255.0, blue: 136.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magenta (Crayola), whose RBG values are (246, 100, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magentaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 246.0/255.0, green: 100.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magenta (Dye), whose RBG values are (202, 31, 123), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magentaDyeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 202.0/255.0, green: 31.0/255.0, blue: 123.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magenta (Pantone), whose RBG values are (208, 65, 126), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magentaPantoneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 208.0/255.0, green: 65.0/255.0, blue: 126.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magenta (Process), whose RBG values are (255, 0, 144), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magentaProcessColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 144.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magenta Haze, whose RBG values are (159, 69, 118), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magentaHazeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 159.0/255.0, green: 69.0/255.0, blue: 118.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magenta-Pink, whose RBG values are (204, 51, 139), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magentaPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 51.0/255.0, blue: 139.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magic Mint (Crayola), whose RBG values are (170, 240, 209), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magicMintCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 170.0/255.0, green: 240.0/255.0, blue: 209.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Magnolia, whose RBG values are (248, 244, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func magnoliaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 244.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mahogany, whose RBG values are (192, 64, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mahoganyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 192.0/255.0, green: 64.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mahogany (Crayola), whose RBG values are (205, 74, 76), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mahoganyCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 74.0/255.0, blue: 76.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Maize, whose RBG values are (251, 236, 93), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func maizeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 236.0/255.0, blue: 93.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Maize (Crayola), whose RBG values are (237, 209, 156), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func maizeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 209.0/255.0, blue: 156.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Majorelle Blue, whose RBG values are (96, 80, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func majorelleBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 96.0/255.0, green: 80.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Malachite, whose RBG values are (11, 218, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func malachiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 11.0/255.0, green: 218.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Manatee (Crayola), whose RBG values are (151, 154, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func manateeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 151.0/255.0, green: 154.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mango Tango (Crayola), whose RBG values are (255, 130, 67), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mangoTangoCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 130.0/255.0, blue: 67.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mantis, whose RBG values are (116, 195, 101), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mantisColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 116.0/255.0, green: 195.0/255.0, blue: 101.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mardi Gras, whose RBG values are (136, 0, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mardiGrasColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 0.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Marigold, whose RBG values are (234, 162, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func marigoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 234.0/255.0, green: 162.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Maroon (Crayola), whose RBG values are (200, 56, 90), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func maroonCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 200.0/255.0, green: 56.0/255.0, blue: 90.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Maroon (HTML/CSS), whose RBG values are (128, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func maroonHTMLCSSColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Maroon (X11), whose RBG values are (176, 48, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func maroonX11Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 48.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mauve, whose RBG values are (224, 176, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mauveColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 176.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mauve Taupe, whose RBG values are (145, 95, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mauveTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 145.0/255.0, green: 95.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mauvelous (Crayola), whose RBG values are (239, 152, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mauvelousCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 152.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color May Green, whose RBG values are (76, 145, 65), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mayGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 76.0/255.0, green: 145.0/255.0, blue: 65.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Maya Blue, whose RBG values are (115, 194, 251), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mayaBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 115.0/255.0, green: 194.0/255.0, blue: 251.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Meat Brown, whose RBG values are (229, 183, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func meatBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 183.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Persian Blue, whose RBG values are (0, 103, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumPersianBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 103.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Tuscan Red, whose RBG values are (121, 68, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumTuscanRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 121.0/255.0, green: 68.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Aquamarine, whose RBG values are (102, 221, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumAquamarineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 221.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Blue, whose RBG values are (0, 0, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 0.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Candy Apple Red, whose RBG values are (226, 6, 44), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumCandyAppleRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 226.0/255.0, green: 6.0/255.0, blue: 44.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Carmine, whose RBG values are (175, 64, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumCarmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 175.0/255.0, green: 64.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Champagne, whose RBG values are (243, 229, 171), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumChampagneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 243.0/255.0, green: 229.0/255.0, blue: 171.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Electric Blue, whose RBG values are (3, 80, 150), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumElectricBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 3.0/255.0, green: 80.0/255.0, blue: 150.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Jungle Green, whose RBG values are (28, 53, 45), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumJungleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 28.0/255.0, green: 53.0/255.0, blue: 45.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Lavender Magenta, whose RBG values are (221, 160, 221), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumLavenderMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 221.0/255.0, green: 160.0/255.0, blue: 221.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Orchid, whose RBG values are (186, 85, 211), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumOrchidColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 186.0/255.0, green: 85.0/255.0, blue: 211.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Purple, whose RBG values are (147, 112, 219), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 147.0/255.0, green: 112.0/255.0, blue: 219.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Red-Violet, whose RBG values are (187, 51, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumRedVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 187.0/255.0, green: 51.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Ruby, whose RBG values are (170, 64, 105), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumRubyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 170.0/255.0, green: 64.0/255.0, blue: 105.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Sea Green, whose RBG values are (60, 179, 113), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumSeaGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 60.0/255.0, green: 179.0/255.0, blue: 113.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Sky Blue, whose RBG values are (128, 218, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 218.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Slate Blue, whose RBG values are (123, 104, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumSlateBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 123.0/255.0, green: 104.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Spring Bud, whose RBG values are (201, 220, 135), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumSpringBudColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 201.0/255.0, green: 220.0/255.0, blue: 135.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Spring Green, whose RBG values are (0, 250, 154), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumSpringGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 250.0/255.0, blue: 154.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Taupe, whose RBG values are (103, 76, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 103.0/255.0, green: 76.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Turquoise, whose RBG values are (72, 209, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumTurquoiseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 72.0/255.0, green: 209.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Vermilion, whose RBG values are (217, 96, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumVermilionColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 217.0/255.0, green: 96.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Medium Violet-Red, whose RBG values are (199, 21, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mediumVioletRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 199.0/255.0, green: 21.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mellow Apricot, whose RBG values are (248, 184, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mellowApricotColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 184.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mellow Yellow, whose RBG values are (248, 222, 126), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mellowYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 222.0/255.0, blue: 126.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Melon (Crayola), whose RBG values are (253, 188, 180), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func melonCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 188.0/255.0, blue: 180.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Metallic Seaweed, whose RBG values are (10, 126, 140), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func metallicSeaweedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 10.0/255.0, green: 126.0/255.0, blue: 140.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Metallic Sunburst, whose RBG values are (156, 124, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func metallicSunburstColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 156.0/255.0, green: 124.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mexican Pink, whose RBG values are (228, 0, 124), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mexicanPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 0.0/255.0, blue: 124.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Midnight Blue (Crayola), whose RBG values are (26, 72, 118), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func midnightBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 26.0/255.0, green: 72.0/255.0, blue: 118.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Midnight Blue, whose RBG values are (25, 25, 112), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func midnightBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 25.0/255.0, green: 25.0/255.0, blue: 112.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Midnight Green, whose RBG values are (0, 73, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func midnightGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 73.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mikado Yellow, whose RBG values are (255, 196, 12), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mikadoYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 196.0/255.0, blue: 12.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mindaro, whose RBG values are (227, 249, 136), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mindaroColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 249.0/255.0, blue: 136.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ming, whose RBG values are (54, 116, 125), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mingColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 54.0/255.0, green: 116.0/255.0, blue: 125.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mint, whose RBG values are (62, 180, 137), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mintColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 62.0/255.0, green: 180.0/255.0, blue: 137.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mint Cream, whose RBG values are (245, 255, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mintCreamColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 255.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mint Green, whose RBG values are (152, 255, 152), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mintGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 255.0/255.0, blue: 152.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Misty Rose, whose RBG values are (255, 228, 225), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mistyRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 228.0/255.0, blue: 225.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Moccasin, whose RBG values are (250, 235, 215), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func moccasinColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 235.0/255.0, blue: 215.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mode Beige, whose RBG values are (150, 113, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func modeBeigeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 113.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Moonstone Blue, whose RBG values are (115, 169, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func moonstoneBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 115.0/255.0, green: 169.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mordant Red 19, whose RBG values are (174, 12, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mordantRed19Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 174.0/255.0, green: 12.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Moss Green, whose RBG values are (138, 154, 91), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mossGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 138.0/255.0, green: 154.0/255.0, blue: 91.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mountain Meadow, whose RBG values are (48, 186, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mountainMeadowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 48.0/255.0, green: 186.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mountain Meadow (Crayola), whose RBG values are (48, 186, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mountainMeadowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 48.0/255.0, green: 186.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mountbatten Pink, whose RBG values are (153, 122, 141), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mountbattenPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 122.0/255.0, blue: 141.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mughal Green, whose RBG values are (48, 96, 48), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mughalGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 48.0/255.0, green: 96.0/255.0, blue: 48.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mulberry (Crayola), whose RBG values are (197, 75, 140), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mulberryCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 197.0/255.0, green: 75.0/255.0, blue: 140.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Mustard, whose RBG values are (255, 219, 88), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func mustardColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 219.0/255.0, blue: 88.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Myrtle Green, whose RBG values are (49, 120, 115), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func myrtleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 49.0/255.0, green: 120.0/255.0, blue: 115.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Nadeshiko Pink, whose RBG values are (246, 173, 198), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func nadeshikoPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 246.0/255.0, green: 173.0/255.0, blue: 198.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Napier Green, whose RBG values are (42, 128, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func napierGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 42.0/255.0, green: 128.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Naples Yellow, whose RBG values are (250, 218, 94), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func naplesYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 218.0/255.0, blue: 94.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Navajo White, whose RBG values are (255, 222, 173), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func navajoWhiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 222.0/255.0, blue: 173.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Navy, whose RBG values are (0, 0, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func navyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 0.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Navy Blue, whose RBG values are (0, 0, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func navyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 0.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Navy Blue (Crayola), whose RBG values are (25, 116, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func navyBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 25.0/255.0, green: 116.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Navy Purple, whose RBG values are (148, 87, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func navyPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 148.0/255.0, green: 87.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Neon Carrot (Crayola), whose RBG values are (255, 163, 67), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func neonCarrotCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 163.0/255.0, blue: 67.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Neon Fuchsia, whose RBG values are (254, 65, 100), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func neonFuchsiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 65.0/255.0, blue: 100.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Neon Green, whose RBG values are (57, 255, 20), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func neonGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 57.0/255.0, green: 255.0/255.0, blue: 20.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color New Car, whose RBG values are (33, 79, 198), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func newCarColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 33.0/255.0, green: 79.0/255.0, blue: 198.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color New York Pink, whose RBG values are (215, 131, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func newYorkPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 131.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Non-Photo Blue, whose RBG values are (164, 221, 237), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func nonPhotoBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 164.0/255.0, green: 221.0/255.0, blue: 237.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color North Texas Green, whose RBG values are (5, 144, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func northTexasGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 5.0/255.0, green: 144.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Nyanza, whose RBG values are (233, 255, 219), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func nyanzaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 255.0/255.0, blue: 219.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color OU Crimson Red, whose RBG values are (153, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oUCrimsonRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ocean Boat Blue, whose RBG values are (0, 119, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oceanBoatBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 119.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ochre, whose RBG values are (204, 119, 34), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ochreColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 119.0/255.0, blue: 34.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Office Green, whose RBG values are (0, 128, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func officeGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 128.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Burgundy, whose RBG values are (67, 48, 46), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldBurgundyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 67.0/255.0, green: 48.0/255.0, blue: 46.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Gold, whose RBG values are (207, 181, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 207.0/255.0, green: 181.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Heliotrope, whose RBG values are (86, 60, 92), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldHeliotropeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 86.0/255.0, green: 60.0/255.0, blue: 92.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Lace, whose RBG values are (253, 245, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldLaceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 245.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Lavender, whose RBG values are (121, 104, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 121.0/255.0, green: 104.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Mauve, whose RBG values are (103, 49, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldMauveColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 103.0/255.0, green: 49.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Moss Green, whose RBG values are (134, 126, 54), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldMossGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 134.0/255.0, green: 126.0/255.0, blue: 54.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Rose, whose RBG values are (192, 128, 129), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 192.0/255.0, green: 128.0/255.0, blue: 129.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Old Silver, whose RBG values are (132, 132, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oldSilverColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 132.0/255.0, green: 132.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Olive, whose RBG values are (128, 128, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oliveColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 128.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Olive Drab #3, whose RBG values are (107, 142, 35), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oliveDrabNumber3Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 107.0/255.0, green: 142.0/255.0, blue: 35.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Olive Drab #7, whose RBG values are (60, 52, 31), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oliveDrabNumber7Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 60.0/255.0, green: 52.0/255.0, blue: 31.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Olive Green (Crayola), whose RBG values are (186, 184, 108), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oliveGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 186.0/255.0, green: 184.0/255.0, blue: 108.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Olivine, whose RBG values are (154, 185, 115), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func olivineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 154.0/255.0, green: 185.0/255.0, blue: 115.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Onyx, whose RBG values are (53, 56, 57), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func onyxColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 53.0/255.0, green: 56.0/255.0, blue: 57.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Opera Mauve, whose RBG values are (183, 132, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func operaMauveColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 183.0/255.0, green: 132.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange (Color Wheel), whose RBG values are (255, 127, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeColorWheelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 127.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange (Crayola), whose RBG values are (255, 117, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 117.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange (Pantone), whose RBG values are (255, 88, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangePantoneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 88.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange (RYB), whose RBG values are (251, 153, 2), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeRYBColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 153.0/255.0, blue: 2.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange (Web), whose RBG values are (255, 165, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 165.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 100, whose RBG values are (255, 224, 178), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 224.0/255.0, blue: 178.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 200, whose RBG values are (255, 204, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 204.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 300, whose RBG values are (255, 183, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 183.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 400, whose RBG values are (255, 167, 38), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 167.0/255.0, blue: 38.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 50, whose RBG values are (255, 243, 224), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 243.0/255.0, blue: 224.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 500, whose RBG values are (255, 152, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 152.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 600, whose RBG values are (251, 140, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 140.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 700, whose RBG values are (245, 124, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 124.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 800, whose RBG values are (239, 108, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 108.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange 900, whose RBG values are (230, 81, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orange900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 81.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange A100, whose RBG values are (255, 209, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 209.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange A200, whose RBG values are (255, 171, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 171.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange A400, whose RBG values are (255, 145, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 145.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange A700, whose RBG values are (255, 109, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 109.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange Red (Crayola), whose RBG values are (255, 43, 43), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeRedCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 43.0/255.0, blue: 43.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange Yellow (Crayola), whose RBG values are (248, 213, 104), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeYellowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 213.0/255.0, blue: 104.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange Peel, whose RBG values are (255, 159, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangePeelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 159.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orange-Red, whose RBG values are (255, 69, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orangeRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 69.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orchid, whose RBG values are (218, 112, 214), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orchidColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 112.0/255.0, blue: 214.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orchid (Crayola), whose RBG values are (230, 168, 215), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orchidCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 168.0/255.0, blue: 215.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orchid Pink, whose RBG values are (242, 189, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func orchidPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 242.0/255.0, green: 189.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Orioles Orange, whose RBG values are (251, 79, 20), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oriolesOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 79.0/255.0, blue: 20.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Otter Brown, whose RBG values are (101, 67, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func otterBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 101.0/255.0, green: 67.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Outer Space (Crayola), whose RBG values are (65, 74, 76), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func outerSpaceCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 65.0/255.0, green: 74.0/255.0, blue: 76.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Outrageous Orange (Crayola), whose RBG values are (255, 110, 74), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func outrageousOrangeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 110.0/255.0, blue: 74.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Oxford Blue, whose RBG values are (0, 33, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func oxfordBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 33.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pacific Blue (Crayola), whose RBG values are (28, 169, 201), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pacificBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 28.0/255.0, green: 169.0/255.0, blue: 201.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pakistan Green, whose RBG values are (0, 102, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pakistanGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 102.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Palatinate Blue, whose RBG values are (39, 59, 226), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func palatinateBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 39.0/255.0, green: 59.0/255.0, blue: 226.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Palatinate Purple, whose RBG values are (104, 40, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func palatinatePurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 104.0/255.0, green: 40.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Aqua, whose RBG values are (188, 212, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleAquaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 188.0/255.0, green: 212.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Blue, whose RBG values are (175, 238, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 175.0/255.0, green: 238.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Brown, whose RBG values are (152, 118, 84), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 118.0/255.0, blue: 84.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Carmine, whose RBG values are (175, 64, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleCarmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 175.0/255.0, green: 64.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Cerulean, whose RBG values are (155, 196, 226), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleCeruleanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 155.0/255.0, green: 196.0/255.0, blue: 226.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Chestnut, whose RBG values are (221, 173, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleChestnutColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 221.0/255.0, green: 173.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Copper, whose RBG values are (218, 138, 103), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleCopperColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 138.0/255.0, blue: 103.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Cornflower Blue, whose RBG values are (171, 205, 239), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleCornflowerBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 171.0/255.0, green: 205.0/255.0, blue: 239.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Cyan, whose RBG values are (135, 211, 248), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleCyanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 135.0/255.0, green: 211.0/255.0, blue: 248.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Gold, whose RBG values are (230, 190, 138), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 190.0/255.0, blue: 138.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Goldenrod, whose RBG values are (238, 232, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleGoldenrodColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 232.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Green, whose RBG values are (152, 251, 152), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 251.0/255.0, blue: 152.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Lavender, whose RBG values are (220, 208, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 220.0/255.0, green: 208.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Magenta, whose RBG values are (249, 132, 229), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 249.0/255.0, green: 132.0/255.0, blue: 229.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Magenta-Pink, whose RBG values are (255, 153, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleMagentaPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 153.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Pink, whose RBG values are (250, 218, 221), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func palePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 218.0/255.0, blue: 221.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Plum, whose RBG values are (221, 160, 221), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func palePlumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 221.0/255.0, green: 160.0/255.0, blue: 221.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Red-Violet, whose RBG values are (219, 112, 147), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleRedVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 219.0/255.0, green: 112.0/255.0, blue: 147.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Robin Egg Blue, whose RBG values are (150, 222, 209), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleRobinEggBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 222.0/255.0, blue: 209.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Silver, whose RBG values are (201, 192, 187), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleSilverColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 201.0/255.0, green: 192.0/255.0, blue: 187.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Spring Bud, whose RBG values are (236, 235, 189), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleSpringBudColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 236.0/255.0, green: 235.0/255.0, blue: 189.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Taupe, whose RBG values are (188, 152, 126), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 188.0/255.0, green: 152.0/255.0, blue: 126.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Turquoise, whose RBG values are (175, 238, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleTurquoiseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 175.0/255.0, green: 238.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Violet, whose RBG values are (204, 153, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 153.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pale Violet-Red, whose RBG values are (219, 112, 147), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paleVioletRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 219.0/255.0, green: 112.0/255.0, blue: 147.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pansy Purple, whose RBG values are (120, 24, 74), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pansyPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 120.0/255.0, green: 24.0/255.0, blue: 74.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Paolo Veronese Green, whose RBG values are (0, 155, 125), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paoloVeroneseGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 155.0/255.0, blue: 125.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Papaya Whip, whose RBG values are (255, 239, 213), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func papayaWhipColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 239.0/255.0, blue: 213.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Paradise Pink, whose RBG values are (230, 62, 98), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paradisePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 62.0/255.0, blue: 98.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Paris Green, whose RBG values are (80, 200, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func parisGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 80.0/255.0, green: 200.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Blue, whose RBG values are (174, 198, 207), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 174.0/255.0, green: 198.0/255.0, blue: 207.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Brown, whose RBG values are (130, 105, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 130.0/255.0, green: 105.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Gray, whose RBG values are (207, 207, 196), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 207.0/255.0, green: 207.0/255.0, blue: 196.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Green, whose RBG values are (119, 221, 119), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 119.0/255.0, green: 221.0/255.0, blue: 119.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Magenta, whose RBG values are (244, 154, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 154.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Orange, whose RBG values are (255, 179, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 179.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Pink, whose RBG values are (222, 165, 164), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 165.0/255.0, blue: 164.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Purple, whose RBG values are (179, 158, 181), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 158.0/255.0, blue: 181.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Red, whose RBG values are (255, 105, 97), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 105.0/255.0, blue: 97.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Violet, whose RBG values are (203, 153, 201), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 203.0/255.0, green: 153.0/255.0, blue: 201.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pastel Yellow, whose RBG values are (253, 253, 150), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pastelYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 253.0/255.0, blue: 150.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Patriarch, whose RBG values are (128, 0, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func patriarchColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 0.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Payne's Grey, whose RBG values are (83, 104, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func paynesGreyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 83.0/255.0, green: 104.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Peach, whose RBG values are (255, 229, 180), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func peachColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 229.0/255.0, blue: 180.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Peach (Crayola), whose RBG values are (255, 207, 171), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func peachCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 207.0/255.0, blue: 171.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Peach Puff, whose RBG values are (255, 218, 185), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func peachPuffColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 218.0/255.0, blue: 185.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Peach-Orange, whose RBG values are (255, 204, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func peachOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 204.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Peach-Yellow, whose RBG values are (250, 223, 173), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func peachYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 223.0/255.0, blue: 173.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pear, whose RBG values are (209, 226, 49), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pearColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 209.0/255.0, green: 226.0/255.0, blue: 49.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pearl, whose RBG values are (234, 224, 200), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pearlColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 234.0/255.0, green: 224.0/255.0, blue: 200.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pearl Aqua, whose RBG values are (136, 216, 192), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pearlAquaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 216.0/255.0, blue: 192.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pearly Purple, whose RBG values are (183, 104, 162), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pearlyPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 183.0/255.0, green: 104.0/255.0, blue: 162.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Peridot, whose RBG values are (230, 226, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func peridotColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 226.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Periwinkle, whose RBG values are (204, 204, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func periwinkleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 204.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Periwinkle (Crayola), whose RBG values are (197, 208, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func periwinkleCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 197.0/255.0, green: 208.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Blue, whose RBG values are (28, 57, 187), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 28.0/255.0, green: 57.0/255.0, blue: 187.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Green, whose RBG values are (0, 166, 147), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 166.0/255.0, blue: 147.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Indigo, whose RBG values are (50, 18, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianIndigoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 50.0/255.0, green: 18.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Orange, whose RBG values are (217, 144, 88), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 217.0/255.0, green: 144.0/255.0, blue: 88.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Pink, whose RBG values are (247, 127, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 127.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Plum, whose RBG values are (112, 28, 28), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianPlumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 28.0/255.0, blue: 28.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Red, whose RBG values are (204, 51, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 51.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persian Rose, whose RBG values are (254, 40, 162), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persianRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 40.0/255.0, blue: 162.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Persimmon, whose RBG values are (236, 88, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func persimmonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 236.0/255.0, green: 88.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Peru, whose RBG values are (205, 133, 63), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func peruColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 133.0/255.0, blue: 63.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Phlox, whose RBG values are (223, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func phloxColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 223.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Phthalo Blue, whose RBG values are (0, 15, 137), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func phthaloBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 15.0/255.0, blue: 137.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Phthalo Green, whose RBG values are (18, 53, 36), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func phthaloGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 18.0/255.0, green: 53.0/255.0, blue: 36.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Picton Blue, whose RBG values are (69, 177, 232), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pictonBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 69.0/255.0, green: 177.0/255.0, blue: 232.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pictorial Carmine, whose RBG values are (195, 11, 78), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pictorialCarmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 195.0/255.0, green: 11.0/255.0, blue: 78.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Piggy Pink (Crayola), whose RBG values are (253, 221, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func piggyPinkCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 221.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pine Green (Crayola), whose RBG values are (21, 128, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pineGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 21.0/255.0, green: 128.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pine Green, whose RBG values are (1, 121, 111), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pineGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 1.0/255.0, green: 121.0/255.0, blue: 111.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pineapple, whose RBG values are (86, 60, 13), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pineappleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 86.0/255.0, green: 60.0/255.0, blue: 13.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink, whose RBG values are (255, 192, 203), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 192.0/255.0, blue: 203.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink (Pantone), whose RBG values are (215, 72, 148), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkPantoneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 72.0/255.0, blue: 148.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 100, whose RBG values are (248, 187, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 187.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 200, whose RBG values are (244, 143, 177), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 143.0/255.0, blue: 177.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 300, whose RBG values are (240, 98, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 98.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 400, whose RBG values are (236, 64, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 236.0/255.0, green: 64.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 50, whose RBG values are (252, 228, 236), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 228.0/255.0, blue: 236.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 500, whose RBG values are (233, 30, 99), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 233.0/255.0, green: 30.0/255.0, blue: 99.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 600, whose RBG values are (216, 27, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 216.0/255.0, green: 27.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 700, whose RBG values are (194, 24, 91), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 194.0/255.0, green: 24.0/255.0, blue: 91.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 800, whose RBG values are (173, 20, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 173.0/255.0, green: 20.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink 900, whose RBG values are (136, 14, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pink900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 14.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink A100, whose RBG values are (255, 128, 171), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 128.0/255.0, blue: 171.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink A200, whose RBG values are (255, 64, 129), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 64.0/255.0, blue: 129.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink A400, whose RBG values are (245, 0, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 0.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink A700, whose RBG values are (197, 17, 98), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 197.0/255.0, green: 17.0/255.0, blue: 98.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink Flamingo (Crayola), whose RBG values are (252, 116, 253), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkFlamingoCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 116.0/255.0, blue: 253.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink Sherbert (Crayola), whose RBG values are (247, 143, 167), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkSherbertCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 143.0/255.0, blue: 167.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink Lace, whose RBG values are (255, 221, 244), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkLaceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 221.0/255.0, blue: 244.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink Lavender, whose RBG values are (216, 178, 209), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 216.0/255.0, green: 178.0/255.0, blue: 209.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink Pearl, whose RBG values are (231, 172, 207), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkPearlColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 231.0/255.0, green: 172.0/255.0, blue: 207.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink Raspberry, whose RBG values are (152, 0, 54), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkRaspberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 0.0/255.0, blue: 54.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pink-Orange, whose RBG values are (255, 153, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pinkOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 153.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pistachio, whose RBG values are (147, 197, 114), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pistachioColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 147.0/255.0, green: 197.0/255.0, blue: 114.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Platinum, whose RBG values are (229, 228, 226), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func platinumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 228.0/255.0, blue: 226.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Plum, whose RBG values are (142, 69, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func plumColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 142.0/255.0, green: 69.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Plum (Crayola), whose RBG values are (142, 69, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func plumCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 142.0/255.0, green: 69.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Plum (Web), whose RBG values are (221, 160, 221), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func plumWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 221.0/255.0, green: 160.0/255.0, blue: 221.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pomp And Power, whose RBG values are (134, 96, 142), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pompAndPowerColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 134.0/255.0, green: 96.0/255.0, blue: 142.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Popstar, whose RBG values are (190, 79, 98), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func popstarColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 190.0/255.0, green: 79.0/255.0, blue: 98.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Portland Orange, whose RBG values are (255, 90, 54), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func portlandOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 90.0/255.0, blue: 54.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Powder Blue, whose RBG values are (176, 224, 230), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func powderBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 224.0/255.0, blue: 230.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Princeton Orange, whose RBG values are (245, 128, 37), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func princetonOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 128.0/255.0, blue: 37.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Prune, whose RBG values are (112, 28, 28), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pruneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 28.0/255.0, blue: 28.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Prussian Blue, whose RBG values are (0, 49, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func prussianBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 49.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Psychedelic Purple, whose RBG values are (223, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func psychedelicPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 223.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Puce, whose RBG values are (204, 136, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func puceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 136.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Puce Red, whose RBG values are (114, 47, 55), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func puceRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 114.0/255.0, green: 47.0/255.0, blue: 55.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pullman Brown (UPS Brown), whose RBG values are (100, 65, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pullmanBrownUPSBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 100.0/255.0, green: 65.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pullman Green, whose RBG values are (59, 51, 28), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pullmanGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 59.0/255.0, green: 51.0/255.0, blue: 28.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Pumpkin, whose RBG values are (255, 117, 24), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func pumpkinColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 117.0/255.0, blue: 24.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple (HTML), whose RBG values are (128, 0, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleHTMLColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 0.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple (Munsell), whose RBG values are (159, 0, 197), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleMunsellColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 159.0/255.0, green: 0.0/255.0, blue: 197.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple (X11), whose RBG values are (160, 32, 240), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleX11Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 160.0/255.0, green: 32.0/255.0, blue: 240.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 100, whose RBG values are (225, 190, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 225.0/255.0, green: 190.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 200, whose RBG values are (206, 147, 216), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 206.0/255.0, green: 147.0/255.0, blue: 216.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 300, whose RBG values are (186, 104, 200), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 186.0/255.0, green: 104.0/255.0, blue: 200.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 400, whose RBG values are (171, 71, 188), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 171.0/255.0, green: 71.0/255.0, blue: 188.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 50, whose RBG values are (243, 229, 245), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 243.0/255.0, green: 229.0/255.0, blue: 245.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 500, whose RBG values are (156, 39, 176), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 156.0/255.0, green: 39.0/255.0, blue: 176.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 600, whose RBG values are (142, 36, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 142.0/255.0, green: 36.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 700, whose RBG values are (123, 31, 162), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 123.0/255.0, green: 31.0/255.0, blue: 162.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 800, whose RBG values are (106, 27, 154), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 106.0/255.0, green: 27.0/255.0, blue: 154.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple 900, whose RBG values are (74, 20, 140), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purple900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 74.0/255.0, green: 20.0/255.0, blue: 140.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple A100, whose RBG values are (234, 128, 252), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 234.0/255.0, green: 128.0/255.0, blue: 252.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple A200, whose RBG values are (224, 64, 251), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 64.0/255.0, blue: 251.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple A400, whose RBG values are (213, 0, 249), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 213.0/255.0, green: 0.0/255.0, blue: 249.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple A700, whose RBG values are (170, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 170.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple Heart, whose RBG values are (105, 53, 156), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleHeartColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 105.0/255.0, green: 53.0/255.0, blue: 156.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple Heart (Crayola), whose RBG values are (116, 66, 200), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleHeartCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 116.0/255.0, green: 66.0/255.0, blue: 200.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple Mountains' Majesty (Crayola), whose RBG values are (157, 129, 186), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleMountainsMajestyCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 157.0/255.0, green: 129.0/255.0, blue: 186.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple Pizzazz (Crayola), whose RBG values are (254, 78, 218), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purplePizzazzCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 78.0/255.0, blue: 218.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple Mountain Majesty, whose RBG values are (150, 120, 182), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleMountainMajestyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 120.0/255.0, blue: 182.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple Navy, whose RBG values are (78, 81, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleNavyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 78.0/255.0, green: 81.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purple Taupe, whose RBG values are (80, 64, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpleTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 80.0/255.0, green: 64.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Purpureus, whose RBG values are (154, 78, 174), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func purpureusColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 154.0/255.0, green: 78.0/255.0, blue: 174.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Quartz, whose RBG values are (81, 72, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func quartzColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 81.0/255.0, green: 72.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Queen Blue, whose RBG values are (67, 107, 149), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func queenBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 67.0/255.0, green: 107.0/255.0, blue: 149.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Queen Pink, whose RBG values are (232, 204, 215), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func queenPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 232.0/255.0, green: 204.0/255.0, blue: 215.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Quinacridone Magenta, whose RBG values are (142, 58, 89), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func quinacridoneMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 142.0/255.0, green: 58.0/255.0, blue: 89.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rackley, whose RBG values are (93, 138, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rackleyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 93.0/255.0, green: 138.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Radical Red, whose RBG values are (255, 53, 94), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func radicalRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 53.0/255.0, blue: 94.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Radical Red (Crayola), whose RBG values are (255, 73, 108), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func radicalRedCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 73.0/255.0, blue: 108.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rajah, whose RBG values are (251, 171, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rajahColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 171.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Raspberry, whose RBG values are (227, 11, 92), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func raspberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 11.0/255.0, blue: 92.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Raspberry Glace, whose RBG values are (145, 95, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func raspberryGlaceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 145.0/255.0, green: 95.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Raspberry Pink, whose RBG values are (226, 80, 152), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func raspberryPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 226.0/255.0, green: 80.0/255.0, blue: 152.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Raspberry Rose, whose RBG values are (179, 68, 108), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func raspberryRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 179.0/255.0, green: 68.0/255.0, blue: 108.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Raw Sienna (Crayola), whose RBG values are (214, 138, 89), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rawSiennaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 214.0/255.0, green: 138.0/255.0, blue: 89.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Raw Umber (Crayola), whose RBG values are (113, 75, 35), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rawUmberCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 113.0/255.0, green: 75.0/255.0, blue: 35.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Raw Umber, whose RBG values are (130, 102, 68), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rawUmberColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 130.0/255.0, green: 102.0/255.0, blue: 68.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Razzle Dazzle Rose (Crayola), whose RBG values are (255, 72, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func razzleDazzleRoseCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 72.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Razzle Dazzle Rose, whose RBG values are (255, 51, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func razzleDazzleRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 51.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Razzmatazz (Crayola), whose RBG values are (227, 37, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func razzmatazzCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 37.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Razzmic Berry, whose RBG values are (141, 78, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func razzmicBerryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 141.0/255.0, green: 78.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rebecca Purple, whose RBG values are (102, 52, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rebeccaPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 52.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red (Crayola), whose RBG values are (238, 32, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 32.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red (Munsell), whose RBG values are (242, 0, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redMunsellColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 242.0/255.0, green: 0.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red (NCS), whose RBG values are (196, 2, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redNCSColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 196.0/255.0, green: 2.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red (Pantone), whose RBG values are (237, 41, 57), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redPantoneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 41.0/255.0, blue: 57.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red (Pigment), whose RBG values are (237, 28, 36), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redPigmentColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 237.0/255.0, green: 28.0/255.0, blue: 36.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red (RYB), whose RBG values are (254, 39, 18), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redRYBColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 39.0/255.0, blue: 18.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 100, whose RBG values are (255, 205, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 205.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 200, whose RBG values are (239, 154, 154), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 154.0/255.0, blue: 154.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 300, whose RBG values are (229, 115, 115), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 115.0/255.0, blue: 115.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 400, whose RBG values are (239, 83, 80), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 83.0/255.0, blue: 80.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 50, whose RBG values are (255, 235, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 235.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 500, whose RBG values are (244, 67, 54), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 67.0/255.0, blue: 54.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 600, whose RBG values are (229, 57, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 57.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 700, whose RBG values are (211, 47, 47), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 211.0/255.0, green: 47.0/255.0, blue: 47.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 800, whose RBG values are (198, 40, 40), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 198.0/255.0, green: 40.0/255.0, blue: 40.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red 900, whose RBG values are (183, 28, 28), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func red900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 183.0/255.0, green: 28.0/255.0, blue: 28.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red A100, whose RBG values are (255, 138, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 138.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red A200, whose RBG values are (255, 82, 82), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 82.0/255.0, blue: 82.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red A400, whose RBG values are (255, 23, 68), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 23.0/255.0, blue: 68.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red A700, whose RBG values are (213, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 213.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red Orange (Crayola), whose RBG values are (255, 83, 73), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redOrangeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 83.0/255.0, blue: 73.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red Violet (Crayola), whose RBG values are (192, 68, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redVioletCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 192.0/255.0, green: 68.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red Devil, whose RBG values are (134, 1, 17), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redDevilColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 134.0/255.0, green: 1.0/255.0, blue: 17.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red-Brown, whose RBG values are (165, 42, 42), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 165.0/255.0, green: 42.0/255.0, blue: 42.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red-Purple, whose RBG values are (228, 0, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 0.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Red-Violet, whose RBG values are (199, 21, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 199.0/255.0, green: 21.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Redwood, whose RBG values are (164, 90, 82), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func redwoodColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 164.0/255.0, green: 90.0/255.0, blue: 82.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Regalia, whose RBG values are (82, 45, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func regaliaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 82.0/255.0, green: 45.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Registration Black, whose RBG values are (0, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func registrationBlackColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Resolution Blue, whose RBG values are (0, 35, 135), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func resolutionBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 35.0/255.0, blue: 135.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rhythm, whose RBG values are (119, 118, 150), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rhythmColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 119.0/255.0, green: 118.0/255.0, blue: 150.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Black, whose RBG values are (0, 64, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richBlackColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 64.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Black (FOGRA29), whose RBG values are (1, 11, 19), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richBlackFOGRA29Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 1.0/255.0, green: 11.0/255.0, blue: 19.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Black (FOGRA39), whose RBG values are (1, 2, 3), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richBlackFOGRA39Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 1.0/255.0, green: 2.0/255.0, blue: 3.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Brilliant Lavender, whose RBG values are (241, 167, 254), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richBrilliantLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 241.0/255.0, green: 167.0/255.0, blue: 254.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Carmine, whose RBG values are (215, 0, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richCarmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 215.0/255.0, green: 0.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Electric Blue, whose RBG values are (8, 146, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richElectricBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 8.0/255.0, green: 146.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Lavender, whose RBG values are (167, 107, 207), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 167.0/255.0, green: 107.0/255.0, blue: 207.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Lilac, whose RBG values are (182, 102, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richLilacColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 182.0/255.0, green: 102.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rich Maroon, whose RBG values are (176, 48, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func richMaroonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 176.0/255.0, green: 48.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rifle Green, whose RBG values are (68, 76, 56), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rifleGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 68.0/255.0, green: 76.0/255.0, blue: 56.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Roast Coffee, whose RBG values are (112, 66, 65), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roastCoffeeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 66.0/255.0, blue: 65.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Robin Egg Blue, whose RBG values are (0, 204, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func robinEggBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 204.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Robin's Egg Blue (Crayola), whose RBG values are (31, 206, 203), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func robinsEggBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 31.0/255.0, green: 206.0/255.0, blue: 203.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rocket Metallic, whose RBG values are (138, 127, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rocketMetallicColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 138.0/255.0, green: 127.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Roman Silver, whose RBG values are (131, 137, 150), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func romanSilverColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 131.0/255.0, green: 137.0/255.0, blue: 150.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose, whose RBG values are (255, 0, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Bonbon, whose RBG values are (249, 66, 158), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseBonbonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 249.0/255.0, green: 66.0/255.0, blue: 158.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Ebony, whose RBG values are (103, 72, 70), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseEbonyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 103.0/255.0, green: 72.0/255.0, blue: 70.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Gold, whose RBG values are (183, 110, 121), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 183.0/255.0, green: 110.0/255.0, blue: 121.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Madder, whose RBG values are (227, 38, 54), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseMadderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 38.0/255.0, blue: 54.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Pink, whose RBG values are (255, 102, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rosePinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 102.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Quartz, whose RBG values are (170, 152, 169), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseQuartzColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 170.0/255.0, green: 152.0/255.0, blue: 169.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Red, whose RBG values are (194, 30, 86), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 194.0/255.0, green: 30.0/255.0, blue: 86.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Taupe, whose RBG values are (144, 93, 93), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 144.0/255.0, green: 93.0/255.0, blue: 93.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rose Vale, whose RBG values are (171, 78, 82), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func roseValeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 171.0/255.0, green: 78.0/255.0, blue: 82.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rosewood, whose RBG values are (101, 0, 11), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rosewoodColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 101.0/255.0, green: 0.0/255.0, blue: 11.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rosso Corsa, whose RBG values are (212, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rossoCorsaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 212.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rosy Brown, whose RBG values are (188, 143, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rosyBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 188.0/255.0, green: 143.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Royal Purple (Crayola), whose RBG values are (120, 81, 169), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func royalPurpleCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 120.0/255.0, green: 81.0/255.0, blue: 169.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Royal Azure, whose RBG values are (0, 56, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func royalAzureColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 56.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Royal Blue, whose RBG values are (65, 105, 225), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func royalBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 65.0/255.0, green: 105.0/255.0, blue: 225.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Royal Blue (Darker), whose RBG values are (0, 35, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func royalBlueDarkerColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 35.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Royal Fuchsia, whose RBG values are (202, 44, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func royalFuchsiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 202.0/255.0, green: 44.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Royal Yellow, whose RBG values are (250, 218, 94), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func royalYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 218.0/255.0, blue: 94.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ruber, whose RBG values are (206, 70, 118), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ruberColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 206.0/255.0, green: 70.0/255.0, blue: 118.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rubine Red, whose RBG values are (209, 0, 86), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rubineRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 209.0/255.0, green: 0.0/255.0, blue: 86.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ruby, whose RBG values are (224, 17, 95), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rubyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 17.0/255.0, blue: 95.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ruby Red, whose RBG values are (155, 17, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rubyRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 155.0/255.0, green: 17.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ruddy, whose RBG values are (255, 0, 40), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ruddyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 40.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ruddy Brown, whose RBG values are (187, 101, 40), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ruddyBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 187.0/255.0, green: 101.0/255.0, blue: 40.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ruddy Pink, whose RBG values are (225, 142, 150), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ruddyPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 225.0/255.0, green: 142.0/255.0, blue: 150.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rufous, whose RBG values are (168, 28, 7), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rufousColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 168.0/255.0, green: 28.0/255.0, blue: 7.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Russet, whose RBG values are (128, 70, 27), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func russetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 70.0/255.0, blue: 27.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Russian Green, whose RBG values are (103, 146, 103), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func russianGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 103.0/255.0, green: 146.0/255.0, blue: 103.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Russian Violet, whose RBG values are (50, 23, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func russianVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 50.0/255.0, green: 23.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rust, whose RBG values are (183, 65, 14), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rustColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 183.0/255.0, green: 65.0/255.0, blue: 14.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Rusty Red, whose RBG values are (218, 44, 67), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func rustyRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 44.0/255.0, blue: 67.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sacramento State Green, whose RBG values are (0, 86, 63), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sacramentoStateGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 86.0/255.0, blue: 63.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Saddle Brown, whose RBG values are (139, 69, 19), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func saddleBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 139.0/255.0, green: 69.0/255.0, blue: 19.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Safety Orange, whose RBG values are (255, 120, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func safetyOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 120.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Safety Orange (Blaze Orange), whose RBG values are (255, 103, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func safetyOrangeBlazeOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 103.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Safety Yellow, whose RBG values are (238, 210, 2), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func safetyYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 210.0/255.0, blue: 2.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Saffron, whose RBG values are (244, 196, 48), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func saffronColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 196.0/255.0, blue: 48.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sage, whose RBG values are (188, 184, 138), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sageColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 188.0/255.0, green: 184.0/255.0, blue: 138.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Salmon, whose RBG values are (250, 128, 114), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func salmonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 128.0/255.0, blue: 114.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Salmon (Crayola), whose RBG values are (255, 155, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func salmonCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 155.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Salmon Pink, whose RBG values are (255, 145, 164), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func salmonPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 145.0/255.0, blue: 164.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sand, whose RBG values are (194, 178, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sandColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 194.0/255.0, green: 178.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sand Dune, whose RBG values are (150, 113, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sandDuneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 113.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sandstorm, whose RBG values are (236, 213, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sandstormColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 236.0/255.0, green: 213.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sandy Brown, whose RBG values are (244, 164, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sandyBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 164.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sandy Taupe, whose RBG values are (150, 113, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sandyTaupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 150.0/255.0, green: 113.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sangria, whose RBG values are (146, 0, 10), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sangriaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 146.0/255.0, green: 0.0/255.0, blue: 10.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sap Green, whose RBG values are (80, 125, 42), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sapGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 80.0/255.0, green: 125.0/255.0, blue: 42.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sapphire, whose RBG values are (15, 82, 186), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sapphireColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 15.0/255.0, green: 82.0/255.0, blue: 186.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sapphire Blue, whose RBG values are (0, 103, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sapphireBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 103.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Satin Sheen Gold, whose RBG values are (203, 161, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func satinSheenGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 203.0/255.0, green: 161.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Scarlet, whose RBG values are (255, 36, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func scarletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 36.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Scarlet (Crayola), whose RBG values are (252, 40, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func scarletCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 40.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Scarlet (Websafe), whose RBG values are (255, 51, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func scarletWebsafeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 51.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Schauss Pink, whose RBG values are (255, 145, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func schaussPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 145.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color School Bus Yellow, whose RBG values are (255, 216, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func schoolBusYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 216.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Screamin' Green (Crayola), whose RBG values are (118, 255, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func screaminGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 118.0/255.0, green: 255.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sea Green (Crayola), whose RBG values are (147, 223, 184), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func seaGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 147.0/255.0, green: 223.0/255.0, blue: 184.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sea Blue, whose RBG values are (0, 105, 148), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func seaBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 105.0/255.0, blue: 148.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sea Green, whose RBG values are (46, 139, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func seaGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 46.0/255.0, green: 139.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Seal Brown, whose RBG values are (50, 20, 20), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sealBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 50.0/255.0, green: 20.0/255.0, blue: 20.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Seashell, whose RBG values are (255, 245, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func seashellColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 245.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Selective Yellow, whose RBG values are (255, 186, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func selectiveYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 186.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sepia, whose RBG values are (112, 66, 20), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sepiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 66.0/255.0, blue: 20.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sepia (Crayola), whose RBG values are (165, 105, 79), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sepiaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 165.0/255.0, green: 105.0/255.0, blue: 79.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shadow (Crayola), whose RBG values are (138, 121, 93), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shadowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 138.0/255.0, green: 121.0/255.0, blue: 93.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shadow Blue, whose RBG values are (119, 139, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shadowBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 119.0/255.0, green: 139.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shampoo, whose RBG values are (255, 207, 241), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shampooColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 207.0/255.0, blue: 241.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shamrock (Crayola), whose RBG values are (69, 206, 162), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shamrockCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 69.0/255.0, green: 206.0/255.0, blue: 162.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shamrock Green, whose RBG values are (0, 158, 96), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shamrockGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 158.0/255.0, blue: 96.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sheen Green, whose RBG values are (143, 212, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sheenGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 143.0/255.0, green: 212.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shimmering Blush, whose RBG values are (217, 134, 149), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shimmeringBlushColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 217.0/255.0, green: 134.0/255.0, blue: 149.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shocking Pink (Crayola), whose RBG values are (251, 126, 253), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shockingPinkCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 126.0/255.0, blue: 253.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Shocking Pink, whose RBG values are (252, 15, 192), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func shockingPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 15.0/255.0, blue: 192.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sienna, whose RBG values are (136, 45, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func siennaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 45.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sienna (X11), whose RBG values are (160, 82, 45), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func siennaX11Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 160.0/255.0, green: 82.0/255.0, blue: 45.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Silver, whose RBG values are (192, 192, 192), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func silverColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 192.0/255.0, green: 192.0/255.0, blue: 192.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Silver (Crayola), whose RBG values are (205, 197, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func silverCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 197.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Silver Lake Blue, whose RBG values are (93, 137, 186), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func silverLakeBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 93.0/255.0, green: 137.0/255.0, blue: 186.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Silver Chalice, whose RBG values are (172, 172, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func silverChaliceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 172.0/255.0, green: 172.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Silver Pink, whose RBG values are (196, 174, 173), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func silverPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 196.0/255.0, green: 174.0/255.0, blue: 173.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Silver Sand, whose RBG values are (191, 193, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func silverSandColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 191.0/255.0, green: 193.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sinopia, whose RBG values are (203, 65, 11), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sinopiaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 203.0/255.0, green: 65.0/255.0, blue: 11.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Skobeloff, whose RBG values are (0, 116, 116), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func skobeloffColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 116.0/255.0, blue: 116.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sky Blue (Crayola), whose RBG values are (128, 218, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func skyBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 218.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sky Blue, whose RBG values are (135, 206, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func skyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 135.0/255.0, green: 206.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sky Magenta, whose RBG values are (207, 113, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func skyMagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 207.0/255.0, green: 113.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Slate Blue, whose RBG values are (106, 90, 205), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func slateBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 106.0/255.0, green: 90.0/255.0, blue: 205.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Slate Gray, whose RBG values are (112, 128, 144), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func slateGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 112.0/255.0, green: 128.0/255.0, blue: 144.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Smalt (Dark Powder Blue), whose RBG values are (0, 51, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func smaltDarkPowderBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 51.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Smitten, whose RBG values are (200, 65, 134), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func smittenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 200.0/255.0, green: 65.0/255.0, blue: 134.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Smoke, whose RBG values are (115, 130, 118), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func smokeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 115.0/255.0, green: 130.0/255.0, blue: 118.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Smoky Topaz, whose RBG values are (147, 61, 65), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func smokyTopazColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 147.0/255.0, green: 61.0/255.0, blue: 65.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Smoky Black, whose RBG values are (16, 12, 8), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func smokyBlackColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 16.0/255.0, green: 12.0/255.0, blue: 8.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Snow, whose RBG values are (255, 250, 250), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func snowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 250.0/255.0, blue: 250.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Soap, whose RBG values are (206, 200, 239), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func soapColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 206.0/255.0, green: 200.0/255.0, blue: 239.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Solid Pink, whose RBG values are (137, 56, 67), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func solidPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 137.0/255.0, green: 56.0/255.0, blue: 67.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sonic Silver, whose RBG values are (117, 117, 117), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sonicSilverColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 117.0/255.0, green: 117.0/255.0, blue: 117.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Space Cadet, whose RBG values are (29, 41, 81), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spaceCadetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 29.0/255.0, green: 41.0/255.0, blue: 81.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Bistre, whose RBG values are (128, 117, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishBistreColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 117.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Blue, whose RBG values are (0, 112, 184), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 112.0/255.0, blue: 184.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Carmine, whose RBG values are (209, 0, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishCarmineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 209.0/255.0, green: 0.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Crimson, whose RBG values are (229, 26, 76), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 26.0/255.0, blue: 76.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Gray, whose RBG values are (152, 152, 152), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 152.0/255.0, green: 152.0/255.0, blue: 152.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Green, whose RBG values are (0, 145, 80), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 145.0/255.0, blue: 80.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Orange, whose RBG values are (232, 97, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 232.0/255.0, green: 97.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Pink, whose RBG values are (247, 191, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 191.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Red, whose RBG values are (230, 0, 38), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 230.0/255.0, green: 0.0/255.0, blue: 38.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Sky Blue, whose RBG values are (0, 255, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Violet, whose RBG values are (76, 40, 130), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 76.0/255.0, green: 40.0/255.0, blue: 130.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spanish Viridian, whose RBG values are (0, 127, 92), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spanishViridianColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 127.0/255.0, blue: 92.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spartan Crimson, whose RBG values are (158, 19, 22), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spartanCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 158.0/255.0, green: 19.0/255.0, blue: 22.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spicy Mix, whose RBG values are (139, 95, 77), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spicyMixColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 139.0/255.0, green: 95.0/255.0, blue: 77.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spiro Disco Ball, whose RBG values are (15, 192, 252), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func spiroDiscoBallColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 15.0/255.0, green: 192.0/255.0, blue: 252.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spring Green (Crayola), whose RBG values are (236, 234, 190), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func springGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 236.0/255.0, green: 234.0/255.0, blue: 190.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spring Bud, whose RBG values are (167, 252, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func springBudColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 167.0/255.0, green: 252.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Spring Green, whose RBG values are (0, 255, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func springGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color St Patrick's Blue, whose RBG values are (35, 41, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func stPatricksBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 35.0/255.0, green: 41.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Star Command Blue, whose RBG values are (0, 123, 184), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func starCommandBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 123.0/255.0, blue: 184.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Steel Blue, whose RBG values are (70, 130, 180), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func steelBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 70.0/255.0, green: 130.0/255.0, blue: 180.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Steel Pink, whose RBG values are (204, 51, 204), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func steelPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 51.0/255.0, blue: 204.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Stil De Grain Yellow, whose RBG values are (250, 218, 94), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func stilDeGrainYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 218.0/255.0, blue: 94.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Stizza, whose RBG values are (153, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func stizzaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Stormcloud, whose RBG values are (79, 102, 106), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func stormcloudColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 79.0/255.0, green: 102.0/255.0, blue: 106.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Straw, whose RBG values are (228, 217, 111), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func strawColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 217.0/255.0, blue: 111.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Strawberry, whose RBG values are (252, 90, 141), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func strawberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 90.0/255.0, blue: 141.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sunglow, whose RBG values are (255, 204, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sunglowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 204.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sunglow (Crayola), whose RBG values are (255, 207, 72), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sunglowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 207.0/255.0, blue: 72.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sunray, whose RBG values are (227, 171, 87), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sunrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 171.0/255.0, blue: 87.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sunset, whose RBG values are (250, 214, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sunsetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 214.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Sunset Orange (Crayola), whose RBG values are (253, 94, 83), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func sunsetOrangeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 94.0/255.0, blue: 83.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Super Pink, whose RBG values are (207, 107, 169), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func superPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 207.0/255.0, green: 107.0/255.0, blue: 169.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tan, whose RBG values are (210, 180, 140), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 210.0/255.0, green: 180.0/255.0, blue: 140.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tan (Crayola), whose RBG values are (250, 167, 108), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tanCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 167.0/255.0, blue: 108.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tangelo, whose RBG values are (249, 77, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tangeloColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 249.0/255.0, green: 77.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tangerine, whose RBG values are (242, 133, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tangerineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 242.0/255.0, green: 133.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tangerine Yellow, whose RBG values are (255, 204, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tangerineYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 204.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tango Pink, whose RBG values are (228, 113, 122), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tangoPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 228.0/255.0, green: 113.0/255.0, blue: 122.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Taupe, whose RBG values are (72, 60, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func taupeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 72.0/255.0, green: 60.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Taupe Gray, whose RBG values are (139, 133, 137), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func taupeGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 139.0/255.0, green: 133.0/255.0, blue: 137.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tea Green, whose RBG values are (208, 240, 192), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teaGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 208.0/255.0, green: 240.0/255.0, blue: 192.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tea Rose, whose RBG values are (248, 131, 121), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teaRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 248.0/255.0, green: 131.0/255.0, blue: 121.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tea Rose (Alternate), whose RBG values are (244, 194, 194), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teaRoseAlternateColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 244.0/255.0, green: 194.0/255.0, blue: 194.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal, whose RBG values are (0, 128, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 128.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 100, whose RBG values are (178, 223, 219), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 178.0/255.0, green: 223.0/255.0, blue: 219.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 200, whose RBG values are (128, 203, 196), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 203.0/255.0, blue: 196.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 300, whose RBG values are (77, 182, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 77.0/255.0, green: 182.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 400, whose RBG values are (38, 166, 154), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 38.0/255.0, green: 166.0/255.0, blue: 154.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 50, whose RBG values are (224, 242, 241), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 242.0/255.0, blue: 241.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 500, whose RBG values are (0, 150, 136), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 150.0/255.0, blue: 136.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 600, whose RBG values are (0, 137, 123), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 137.0/255.0, blue: 123.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 700, whose RBG values are (0, 121, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 121.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 800, whose RBG values are (0, 105, 92), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 105.0/255.0, blue: 92.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal 900, whose RBG values are (0, 77, 64), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func teal900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 77.0/255.0, blue: 64.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal A100, whose RBG values are (167, 255, 235), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 167.0/255.0, green: 255.0/255.0, blue: 235.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal A200, whose RBG values are (100, 255, 218), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 100.0/255.0, green: 255.0/255.0, blue: 218.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal A400, whose RBG values are (29, 233, 182), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 29.0/255.0, green: 233.0/255.0, blue: 182.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal A700, whose RBG values are (0, 191, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 191.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal Blue (Crayola), whose RBG values are (24, 167, 181), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 24.0/255.0, green: 167.0/255.0, blue: 181.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal Blue, whose RBG values are (54, 117, 136), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 54.0/255.0, green: 117.0/255.0, blue: 136.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal Deer, whose RBG values are (153, 230, 179), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealDeerColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 230.0/255.0, blue: 179.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Teal Green, whose RBG values are (0, 130, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tealGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 130.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Telemagenta, whose RBG values are (207, 52, 118), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func telemagentaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 207.0/255.0, green: 52.0/255.0, blue: 118.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tenné, whose RBG values are (205, 87, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tennéColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 87.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Terra Cotta, whose RBG values are (226, 114, 91), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func terraCottaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 226.0/255.0, green: 114.0/255.0, blue: 91.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Thistle, whose RBG values are (216, 191, 216), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func thistleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 216.0/255.0, green: 191.0/255.0, blue: 216.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Thistle (Crayola), whose RBG values are (235, 199, 223), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func thistleCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 235.0/255.0, green: 199.0/255.0, blue: 223.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Thulian Pink, whose RBG values are (222, 111, 161), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func thulianPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 111.0/255.0, blue: 161.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tickle Me Pink (Crayola), whose RBG values are (252, 137, 172), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tickleMePinkCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 137.0/255.0, blue: 172.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tiffany Blue, whose RBG values are (10, 186, 181), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tiffanyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 10.0/255.0, green: 186.0/255.0, blue: 181.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tiffany Blue (Alternate), whose RBG values are (129, 216, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tiffanyBlueAlternateColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 129.0/255.0, green: 216.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tiger's Eye, whose RBG values are (224, 141, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tigersEyeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 224.0/255.0, green: 141.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Timberwolf (Crayola), whose RBG values are (219, 215, 210), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func timberwolfCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 219.0/255.0, green: 215.0/255.0, blue: 210.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Titanium Yellow, whose RBG values are (238, 230, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func titaniumYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 230.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tomato, whose RBG values are (255, 99, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tomatoColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 99.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Toolbox, whose RBG values are (116, 108, 192), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func toolboxColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 116.0/255.0, green: 108.0/255.0, blue: 192.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Topaz, whose RBG values are (255, 200, 124), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func topazColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 200.0/255.0, blue: 124.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tractor Red, whose RBG values are (253, 14, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tractorRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 14.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Trolley Grey, whose RBG values are (128, 128, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func trolleyGreyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 128.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tropical Rain Forest (Crayola), whose RBG values are (23, 128, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tropicalRainForestCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 23.0/255.0, green: 128.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tropical Rain Forest, whose RBG values are (0, 117, 94), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tropicalRainForestColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 117.0/255.0, blue: 94.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color True Blue, whose RBG values are (0, 115, 207), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func trueBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 115.0/255.0, blue: 207.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tufts Blue, whose RBG values are (65, 125, 193), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tuftsBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 65.0/255.0, green: 125.0/255.0, blue: 193.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tulip, whose RBG values are (255, 135, 141), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tulipColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 135.0/255.0, blue: 141.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tumbleweed (Crayola), whose RBG values are (222, 170, 136), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tumbleweedCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 222.0/255.0, green: 170.0/255.0, blue: 136.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Turkish Rose, whose RBG values are (181, 114, 129), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func turkishRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 181.0/255.0, green: 114.0/255.0, blue: 129.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Turquoise, whose RBG values are (64, 224, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func turquoiseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 64.0/255.0, green: 224.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Turquoise Blue (Crayola), whose RBG values are (119, 221, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func turquoiseBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 119.0/255.0, green: 221.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Turquoise Blue, whose RBG values are (0, 255, 239), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func turquoiseBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 255.0/255.0, blue: 239.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Turquoise Green, whose RBG values are (160, 214, 180), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func turquoiseGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 160.0/255.0, green: 214.0/255.0, blue: 180.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tuscan, whose RBG values are (250, 214, 165), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tuscanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 250.0/255.0, green: 214.0/255.0, blue: 165.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tuscan Brown, whose RBG values are (111, 78, 55), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tuscanBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 111.0/255.0, green: 78.0/255.0, blue: 55.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tuscan Red, whose RBG values are (124, 72, 72), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tuscanRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 72.0/255.0, blue: 72.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tuscan Tan, whose RBG values are (166, 123, 91), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tuscanTanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 166.0/255.0, green: 123.0/255.0, blue: 91.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tuscany, whose RBG values are (192, 153, 153), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tuscanyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 192.0/255.0, green: 153.0/255.0, blue: 153.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Twilight Lavender, whose RBG values are (138, 73, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func twilightLavenderColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 138.0/255.0, green: 73.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Tyrian Purple, whose RBG values are (102, 2, 60), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func tyrianPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 2.0/255.0, blue: 60.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color UA Blue, whose RBG values are (0, 51, 170), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uABlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 51.0/255.0, blue: 170.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color UA Red, whose RBG values are (217, 0, 76), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uARedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 217.0/255.0, green: 0.0/255.0, blue: 76.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color UCLA Blue, whose RBG values are (83, 104, 149), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uCLABlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 83.0/255.0, green: 104.0/255.0, blue: 149.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color UCLA Gold, whose RBG values are (255, 179, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uCLAGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 179.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color UFO Green, whose RBG values are (60, 208, 112), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uFOGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 60.0/255.0, green: 208.0/255.0, blue: 112.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color UP Forest Green, whose RBG values are (1, 68, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uPForestGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 1.0/255.0, green: 68.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color UP Maroon, whose RBG values are (123, 17, 19), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uPMaroonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 123.0/255.0, green: 17.0/255.0, blue: 19.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color USAFA Blue, whose RBG values are (0, 79, 152), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uSAFABlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 79.0/255.0, blue: 152.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color USC Cardinal, whose RBG values are (153, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uSCCardinalColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 153.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color USC Gold, whose RBG values are (255, 204, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func uSCGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 204.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ube, whose RBG values are (136, 120, 195), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ubeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 136.0/255.0, green: 120.0/255.0, blue: 195.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ultra Pink, whose RBG values are (255, 111, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ultraPinkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 111.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ultra Red, whose RBG values are (252, 108, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ultraRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 108.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ultramarine, whose RBG values are (18, 10, 143), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ultramarineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 18.0/255.0, green: 10.0/255.0, blue: 143.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Ultramarine Blue, whose RBG values are (65, 102, 245), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func ultramarineBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 65.0/255.0, green: 102.0/255.0, blue: 245.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Umber, whose RBG values are (99, 81, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func umberColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 99.0/255.0, green: 81.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Unbleached Silk, whose RBG values are (255, 221, 202), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func unbleachedSilkColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 221.0/255.0, blue: 202.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color United Nations Blue, whose RBG values are (91, 146, 229), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func unitedNationsBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 91.0/255.0, green: 146.0/255.0, blue: 229.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color University Of California Gold, whose RBG values are (183, 135, 39), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func universityOfCaliforniaGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 183.0/255.0, green: 135.0/255.0, blue: 39.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color University Of Tennessee Orange, whose RBG values are (247, 127, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func universityOfTennesseeOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 127.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Unmellow Yellow (Crayola), whose RBG values are (255, 255, 102), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func unmellowYellowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 102.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Upsdell Red, whose RBG values are (174, 32, 41), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func upsdellRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 174.0/255.0, green: 32.0/255.0, blue: 41.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Urobilin, whose RBG values are (225, 173, 33), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func urobilinColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 225.0/255.0, green: 173.0/255.0, blue: 33.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Utah Crimson, whose RBG values are (211, 0, 63), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func utahCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 211.0/255.0, green: 0.0/255.0, blue: 63.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vanilla, whose RBG values are (243, 229, 171), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vanillaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 243.0/255.0, green: 229.0/255.0, blue: 171.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vanilla Ice, whose RBG values are (243, 143, 169), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vanillaIceColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 243.0/255.0, green: 143.0/255.0, blue: 169.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vegas Gold, whose RBG values are (197, 179, 88), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vegasGoldColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 197.0/255.0, green: 179.0/255.0, blue: 88.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Venetian Red, whose RBG values are (200, 8, 21), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func venetianRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 200.0/255.0, green: 8.0/255.0, blue: 21.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Verdigris, whose RBG values are (67, 179, 174), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func verdigrisColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 67.0/255.0, green: 179.0/255.0, blue: 174.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vermilion, whose RBG values are (227, 66, 52), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vermilionColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 227.0/255.0, green: 66.0/255.0, blue: 52.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vermilion (Alternate), whose RBG values are (217, 56, 30), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vermilionAlternateColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 217.0/255.0, green: 56.0/255.0, blue: 30.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Veronica, whose RBG values are (160, 32, 240), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func veronicaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 160.0/255.0, green: 32.0/255.0, blue: 240.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Very Light Azure, whose RBG values are (116, 187, 251), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func veryLightAzureColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 116.0/255.0, green: 187.0/255.0, blue: 251.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Very Light Blue, whose RBG values are (102, 102, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func veryLightBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 102.0/255.0, green: 102.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Very Light Malachite Green, whose RBG values are (100, 233, 134), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func veryLightMalachiteGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 100.0/255.0, green: 233.0/255.0, blue: 134.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Very Light Tangelo, whose RBG values are (255, 176, 119), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func veryLightTangeloColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 176.0/255.0, blue: 119.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Very Pale Orange, whose RBG values are (255, 223, 191), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func veryPaleOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 223.0/255.0, blue: 191.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Very Pale Yellow, whose RBG values are (255, 255, 191), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func veryPaleYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 191.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Violet, whose RBG values are (143, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func violetColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 143.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Violet (Color Wheel), whose RBG values are (127, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func violetColorWheelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 127.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Violet (Purple) (Crayola), whose RBG values are (146, 110, 174), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func violetPurpleCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 146.0/255.0, green: 110.0/255.0, blue: 174.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Violet (RYB), whose RBG values are (134, 1, 175), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func violetRYBColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 134.0/255.0, green: 1.0/255.0, blue: 175.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Violet (Web), whose RBG values are (238, 130, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func violetWebColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 238.0/255.0, green: 130.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Violet Blue (Crayola), whose RBG values are (50, 74, 178), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func violetBlueCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 50.0/255.0, green: 74.0/255.0, blue: 178.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Violet Red (Crayola), whose RBG values are (247, 83, 148), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func violetRedCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 83.0/255.0, blue: 148.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Viridian, whose RBG values are (64, 130, 109), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func viridianColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 64.0/255.0, green: 130.0/255.0, blue: 109.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Viridian Green, whose RBG values are (0, 150, 152), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func viridianGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 150.0/255.0, blue: 152.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vista Blue, whose RBG values are (124, 158, 217), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vistaBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 124.0/255.0, green: 158.0/255.0, blue: 217.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Tangerine (Crayola), whose RBG values are (255, 160, 137), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividTangerineCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 160.0/255.0, blue: 137.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Violet (Crayola), whose RBG values are (143, 80, 157), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividVioletCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 143.0/255.0, green: 80.0/255.0, blue: 157.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Amber, whose RBG values are (204, 153, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividAmberColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 153.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Auburn, whose RBG values are (146, 39, 36), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividAuburnColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 146.0/255.0, green: 39.0/255.0, blue: 36.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Burgundy, whose RBG values are (159, 29, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividBurgundyColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 159.0/255.0, green: 29.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Cerise, whose RBG values are (218, 29, 129), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividCeriseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 218.0/255.0, green: 29.0/255.0, blue: 129.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Cerulean, whose RBG values are (0, 170, 238), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividCeruleanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 170.0/255.0, blue: 238.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Crimson, whose RBG values are (204, 0, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividCrimsonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 0.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Gamboge, whose RBG values are (255, 153, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividGambogeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 153.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Lime Green, whose RBG values are (166, 214, 8), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividLimeGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 166.0/255.0, green: 214.0/255.0, blue: 8.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Malachite, whose RBG values are (0, 204, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividMalachiteColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 204.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Mulberry, whose RBG values are (184, 12, 227), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividMulberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 184.0/255.0, green: 12.0/255.0, blue: 227.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Orange, whose RBG values are (255, 95, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 95.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Orange Peel, whose RBG values are (255, 160, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividOrangePeelColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 160.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Orchid, whose RBG values are (204, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividOrchidColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 204.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Raspberry, whose RBG values are (255, 0, 108), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividRaspberryColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 0.0/255.0, blue: 108.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Red, whose RBG values are (247, 13, 26), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividRedColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 247.0/255.0, green: 13.0/255.0, blue: 26.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Red-Tangelo, whose RBG values are (223, 97, 36), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividRedTangeloColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 223.0/255.0, green: 97.0/255.0, blue: 36.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Sky Blue, whose RBG values are (0, 204, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividSkyBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 204.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Tangelo, whose RBG values are (240, 116, 39), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividTangeloColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 240.0/255.0, green: 116.0/255.0, blue: 39.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Vermilion, whose RBG values are (229, 96, 36), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividVermilionColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 229.0/255.0, green: 96.0/255.0, blue: 36.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Violet, whose RBG values are (159, 0, 255), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividVioletColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 159.0/255.0, green: 0.0/255.0, blue: 255.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Vivid Yellow, whose RBG values are (255, 227, 2), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func vividYellowColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 227.0/255.0, blue: 2.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Volt, whose RBG values are (205, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func voltColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Warm Black, whose RBG values are (0, 66, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func warmBlackColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 66.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Waterspout, whose RBG values are (164, 244, 249), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func waterspoutColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 164.0/255.0, green: 244.0/255.0, blue: 249.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Web Gray, whose RBG values are (128, 128, 128), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func webGrayColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 128.0/255.0, green: 128.0/255.0, blue: 128.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Web Green, whose RBG values are (0, 128, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func webGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 128.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Web Maroon, whose RBG values are (127, 0, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func webMaroonColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 127.0/255.0, green: 0.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Web Purple, whose RBG values are (127, 0, 127), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func webPurpleColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 127.0/255.0, green: 0.0/255.0, blue: 127.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wenge, whose RBG values are (100, 84, 82), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wengeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 100.0/255.0, green: 84.0/255.0, blue: 82.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wheat, whose RBG values are (245, 222, 179), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wheatColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 222.0/255.0, blue: 179.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color White Smoke, whose RBG values are (245, 245, 245), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func whiteSmokeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 245.0/255.0, blue: 245.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wild Blue Yonder (Crayola), whose RBG values are (162, 173, 208), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wildBlueYonderCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 162.0/255.0, green: 173.0/255.0, blue: 208.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wild Strawberry (Crayola), whose RBG values are (255, 67, 164), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wildStrawberryCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 67.0/255.0, blue: 164.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wild Watermelon (Crayola), whose RBG values are (252, 108, 133), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wildWatermelonCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 108.0/255.0, blue: 133.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wild Orchid, whose RBG values are (212, 112, 162), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wildOrchidColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 212.0/255.0, green: 112.0/255.0, blue: 162.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Willpower Orange, whose RBG values are (253, 88, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func willpowerOrangeColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 88.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Windsor Tan, whose RBG values are (167, 85, 2), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func windsorTanColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 167.0/255.0, green: 85.0/255.0, blue: 2.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wine, whose RBG values are (114, 47, 55), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wineColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 114.0/255.0, green: 47.0/255.0, blue: 55.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wine Dregs, whose RBG values are (103, 49, 71), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wineDregsColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 103.0/255.0, green: 49.0/255.0, blue: 71.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wisteria, whose RBG values are (201, 160, 220), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wisteriaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 201.0/255.0, green: 160.0/255.0, blue: 220.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wisteria (Crayola), whose RBG values are (205, 164, 222), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func wisteriaCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 205.0/255.0, green: 164.0/255.0, blue: 222.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Wood Brown, whose RBG values are (193, 154, 107), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func woodBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 193.0/255.0, green: 154.0/255.0, blue: 107.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Xanadu, whose RBG values are (115, 134, 120), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func xanaduColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 115.0/255.0, green: 134.0/255.0, blue: 120.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yale Blue, whose RBG values are (15, 77, 146), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yaleBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 15.0/255.0, green: 77.0/255.0, blue: 146.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yankees Blue, whose RBG values are (28, 40, 65), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yankeesBlueColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 28.0/255.0, green: 40.0/255.0, blue: 65.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow (Crayola), whose RBG values are (252, 232, 131), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 252.0/255.0, green: 232.0/255.0, blue: 131.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow (Munsell), whose RBG values are (239, 204, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowMunsellColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 239.0/255.0, green: 204.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow (NCS), whose RBG values are (255, 211, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowNCSColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 211.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow (Pantone), whose RBG values are (254, 223, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowPantoneColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 223.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow (Process), whose RBG values are (255, 239, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowProcessColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 239.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow (RYB), whose RBG values are (254, 254, 51), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowRYBColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 254.0/255.0, green: 254.0/255.0, blue: 51.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 100, whose RBG values are (255, 249, 196), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 249.0/255.0, blue: 196.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 200, whose RBG values are (255, 245, 157), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 245.0/255.0, blue: 157.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 300, whose RBG values are (255, 241, 118), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow300Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 241.0/255.0, blue: 118.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 400, whose RBG values are (255, 238, 88), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 238.0/255.0, blue: 88.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 50, whose RBG values are (255, 253, 231), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow50Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 253.0/255.0, blue: 231.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 500, whose RBG values are (255, 235, 59), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow500Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 235.0/255.0, blue: 59.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 600, whose RBG values are (253, 216, 53), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow600Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 253.0/255.0, green: 216.0/255.0, blue: 53.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 700, whose RBG values are (251, 192, 45), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 251.0/255.0, green: 192.0/255.0, blue: 45.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 800, whose RBG values are (249, 168, 37), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow800Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 249.0/255.0, green: 168.0/255.0, blue: 37.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow 900, whose RBG values are (245, 127, 23), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellow900Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 245.0/255.0, green: 127.0/255.0, blue: 23.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow A100, whose RBG values are (255, 255, 141), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowA100Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 141.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow A200, whose RBG values are (255, 255, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowA200Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 255.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow A400, whose RBG values are (255, 234, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowA400Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 234.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow A700, whose RBG values are (255, 214, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowA700Color ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 214.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow Green (Crayola), whose RBG values are (197, 227, 132), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowGreenCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 197.0/255.0, green: 227.0/255.0, blue: 132.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow Orange (Crayola), whose RBG values are (255, 174, 66), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowOrangeCrayolaColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 174.0/255.0, blue: 66.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow Green, whose RBG values are (154, 205, 50), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowGreenColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 154.0/255.0, green: 205.0/255.0, blue: 50.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Yellow Rose, whose RBG values are (255, 240, 0), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func yellowRoseColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 255.0/255.0, green: 240.0/255.0, blue: 0.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Zaffre, whose RBG values are (0, 20, 168), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func zaffreColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 0.0/255.0, green: 20.0/255.0, blue: 168.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Zinnwaldite Brown, whose RBG values are (44, 22, 8), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func zinnwalditeBrownColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 44.0/255.0, green: 22.0/255.0, blue: 8.0/255.0, alpha: alpha ) }


    /**
    Returns a NSColor object representing the color Zomp, whose RBG values are (57, 167, 142), and has the specified opacity.

    :param: alpha A CGFloat between 0.0 and 1.0 representing the opacity with a default value of 1.0.

    :returns: The NSColor object
    */

    class func zompColor ( alpha: CGFloat = 1.0 ) -> NSColor {
        return  NSColor.init ( SRGBRed: 57.0/255.0, green: 167.0/255.0, blue: 142.0/255.0, alpha: alpha ) }


}

